﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Threading;

namespace DatabazeParkourovychSpotu
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        // spot = tréninkové místo pro parkour

        #region proměnné, vlastnosti

        private string _query;
        private DataSet _dataSet;
        private DataTable _dataTable;
        public MySqlDatabaze _mySql;
        public SqlCeDatabaze _sqlCe;
        private SplashScreen _splashScreen;
        private int _aktualniIDSpotu = -1;
        private List<Spot> _spoty;
        private List<BitmapImage> _nacteneObrazky = new List<BitmapImage>();
        private bool _sqlCePripojeno = false;
        public bool _splashClose = false;
        public bool _mySqlPripojeno;
        private string _uzivatel = "";
        private string _typZobrazenychSpotu = "";
        private string _typSpotu = "";
        public string TypZobrazenychSpotu
        {
            get { return _typZobrazenychSpotu; }
            set
            {
                _typZobrazenychSpotu = value;
                _spoty = ((_mySqlPripojeno && _typZobrazenychSpotu == "Moje vlastní spoty") || (!_mySqlPripojeno)) ? SqlCeDatabaze.Spoty : MySqlDatabaze.Spoty;

            }
        }
        public int IdUzivatele { get; set; } = 0;
        public string Uzivatel
        {
            get { return _uzivatel; }
            set
            {
                _uzivatel = value;
                if (_uzivatel != "") DokonceniPrihlaseni();
                else DokonceniOdhlaseni();
                NacteniHodnoceniOblibene();
                NaplneniDataGridu();
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(String info)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(info));
        }
        private DataView _dataGridItemsSource;
        public DataView dataGridItemsSource
        {
            get { return _dataGridItemsSource; }
            set
            {
                _dataGridItemsSource = value;
                OnPropertyChanged("dataGridItemsSource");
            }
        }
        #endregion

        #region MainWindow, Inicializace aplikace, nastavení zmáčknutí kláves

        // konstruktor
        // nastaví viditelnost okna 'MainWindow' na Hidden - skryje okno
        // okno je skryté a načte se Splash screen
        // zavolá se metoda pro připojení k databázi
        public MainWindow()
        {
            InitializeComponent();
            this.Visibility = Visibility.Hidden;
            _splashScreen = new SplashScreen(this);
            _splashScreen.Show();
            PripojeniKDatabazi();
        }

        // při zavírání aplikace se zobrazí jestli opravdu chceme zavřít aplikaci
        private void UvodniObrazovka_Closing(object sender, CancelEventArgs e)
        {
            bool result = CustomMessageBox.Show("Opravdu chcete ukončit aplikaci?", "Ukončit aplikaci", "Ano", "Ne");
            if (result == true) e.Cancel = false;
            else e.Cancel = true;
        }

        // nastavuje se chování po stisknutí kláves na klávesnici
        private async void UvodniObrazovka_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            // na hlavní obrazovce
            if (hlavniObrazovkaTabItem.IsSelected && (nazevSpotuTextBox.IsFocused || vyhledavaniCastPrahyComboBox.IsFocused || vyhledavaniPovrchComboBox.IsFocused || vyhledavaniVelikostSpotuComboBox.IsFocused || vyhledavaniTreninkComboBox.IsFocused || vyhledavaniObtiznostComboBox.IsFocused) && ((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return))))
            {
                Keyboard.ClearFocus();
                bool zmenaPripojeni = ZmenaPripojeniInternetu();
                if (!zmenaPripojeni)
                {
                    vyhledavaniButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    vyhledavaniButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 24, 44));
                    vyhledavaniButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 24, 44));
                }

                await Task.Delay(200);

                vyhledavaniButton.ClearValue(Button.BackgroundProperty);
                vyhledavaniButton.ClearValue(Button.BorderBrushProperty);
                return;
            }


            // na obrazovce detailu spotu
            if (detailSpotuTabItem.IsSelected)
                if (e.Key.Equals(Key.Back))
                {
                    prejitNaHlavniStranuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    return;
                }

            // na obrazovce galerie
            if (galerieTabItem.IsSelected)
            {
                if (e.Key.Equals(Key.Back))
                {
                    velkyObrazekPrejitZpetButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    return;
                }
                if (e.Key.Equals(Key.Right) && velkyObrazekNacitaniImage.Opacity == 0)
                {
                    velkyObrazekNasledujiciFotoButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    velkyObrazekNasledujiciFotoButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                    velkyObrazekNasledujiciFotoButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                    await Task.Delay(200);

                    velkyObrazekNasledujiciFotoButton.ClearValue(Button.BackgroundProperty);
                    velkyObrazekNasledujiciFotoButton.ClearValue(Button.BorderBrushProperty);
                    return;
                }
                if (e.Key.Equals(Key.Left) && velkyObrazekNacitaniImage.Opacity == 0)
                {
                    velkyObrazekPredchoziFotoButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    velkyObrazekPredchoziFotoButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                    velkyObrazekPredchoziFotoButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                    await Task.Delay(200);

                    velkyObrazekPredchoziFotoButton.ClearValue(Button.BackgroundProperty);
                    velkyObrazekPredchoziFotoButton.ClearValue(Button.BorderBrushProperty);
                    return;
                }
            }

            // když má button focus a stiskne se enter, tak se provede graficky zmáčknutí buttonu
            foreach (Visual v in hlavniGrid.GetChildren())
            {
                if (v is Button)
                {
                    Button button = (Button)v;
                    if (((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return))) && button.IsFocused)
                    {
                        button.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                        button.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                        await Task.Delay(1000);

                        button.ClearValue(Button.BackgroundProperty);
                        button.ClearValue(Button.BorderBrushProperty);
                    }
                }
            }
        }
        #endregion

        #region Horní Panel s Názvem Aplikace (nazevAplikaceBorder) - Události

        // po kliknutí na button vpravo nahoře se podle textu buttonu provede příslušná operace
        private void nazevAplikaceButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                Button c = (Button)sender;
                Keyboard.ClearFocus();
                if (c.Content.ToString() == "Přihlásit se / Registrovat se") // načte a zobrazí se okno pro přihlášení / registraci
                {
                    PrihlaseniRegistrace prihlaseniRegistraceForm = new PrihlaseniRegistrace(this);
                    prihlaseniRegistraceForm.Show();
                    return;
                }
                if (c.Content.ToString() == "Odhlásit se")    // vymaže přihlášení stávajícího přihlášeného uživatele
                {
                    IdUzivatele = 0;
                    Uzivatel = "";
                    c.Content = "Přihlásit se / Registrovat se";
                    CustomMessageBox.Show("Odhlášení bylo úspěšné.\nNyní jste odhlášení.", "Odhlášení");
                    return;
                }
            }
        }

        // Aplikace se pokusí připojit k MySql databázi
        private void nazevAplikacePripojeniButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                CustomMessageBox.Show("Online databáze stále není k dispozici.\n\nV případě přetrvávajích problémů se obraťte na internetovou stránku www.dpsc.cz.\n\nDěkujeme.", "Připojení");
            }
        }
        #endregion

        #region Menu - Události

        // události pro položky v menu
        private void vyhledavaniNazevMenuItem_Click(object sender, RoutedEventArgs e)
        {
            nazevSpotuTextBox.Focus();
        }

        private void vyhledavaniCastPrahyMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniCastPrahyComboBox.Focus();
            vyhledavaniCastPrahyComboBox.IsDropDownOpen = true;
        }

        private void vyhledavaniPovrchMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniPovrchComboBox.Focus();
            vyhledavaniPovrchComboBox.IsDropDownOpen = true;
        }

        private void vyhledavaniVelikostMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniVelikostSpotuComboBox.Focus();
            vyhledavaniVelikostSpotuComboBox.IsDropDownOpen = true;
        }

        private void vyhledavaniTypMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniTreninkComboBox.Focus();
            vyhledavaniTreninkComboBox.IsDropDownOpen = true;
        }

        private void vyhledavaniObtiznostMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniObtiznostComboBox.Focus();
            vyhledavaniObtiznostComboBox.IsDropDownOpen = true;
        }

        private void vyhledavaniMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vyhledavaniButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void resetovatVyhledavaniMenuItem_Click(object sender, RoutedEventArgs e)
        {
            resetovaniVyhledavaniButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void razeniAZMenuItem_Click(object sender, RoutedEventArgs e)
        {
            razeniAZButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void razeniCastPrahyMenuItem_Click(object sender, RoutedEventArgs e)
        {
            razeniCastPrahyButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void razeniHodnoceniMenuItem_Click(object sender, RoutedEventArgs e)
        {
            razeniHodnoceniButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void razeniVelikostSpotuMenuItem_Click(object sender, RoutedEventArgs e)
        {
            razeniVelikostSpotuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        private void ukoncitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void napovedaMenuItem_Click(object sender, RoutedEventArgs e)
        {
            CustomMessageBox.Show("DATABÁZE PARKOUROVÝCH SPOTŮ V PRAZE\n\nNÁPOVĚDA:\n\nVyhledávání:\nNa hlavní straně po vyplnění formuláře 'Vyhledávání' (možnosti vyhledávání: Podle názvu nebo Pokročilé vyhledávání) klikněte na tlačítko 'Hledat' - v tabulce se aktualizuje seznam podle Vašich kritérií. Kliknutím na tlačítko 'Resetovat vyhledávání' se možnosti vyhledávání resetují a zobrazí se všechny spoty.\n\nŘazení:\nPři kliknutí na příslušné tlačítko na formuláři 'Řazení výsledků' se aktualizuje seznam výsledků.\n\nDetail spotu:\nPři rozkliknutí detailu spotu máte možnost po zvolení příslušné záložky zjistit informace o poloze spotu, dopravě a máte možnost zobrazit fotky, kliknutím na fotku ji zvětšíte.\n\nPřihlášení / Registrace:\nPro přihlášení nebo registraci klikněte v pravém horním rohu na tlačítko 'Přihlásit se / Registrovat se'. Zobrazí se nové okno pro přihlášení / registraci.\n\nFunkce pro registrované uživatele:\nPo přihlášení získáte možnost:\nHodnotit spoty (po rozkliknutí detailu spotu kliknutím na počet hvězdiček, 1 - nejhorší, 5 - nejlepší),\nPřidat spoty do oblíbených (po rozkliknutí detailu spotu kliknutím na tlačítko 'Přidat do oblíbených')\nProhlížet vlastní oblíbené, ohodnocené spoty a vlastní soukromé spoty (na hlavní straně po kliknutí na tlačítka 'Moje oblíbené spoty' / 'Mnou ohodnocené spoty / Moje vlastní spoty')\nPřidávat vlastní spoty (na hlavní straně po kliknutí na tlačítko 'Vytvořit nový vlastní spot')\n\n\nAplikace využívá internetové připojení.\nBez internetového připojení nemusí některé funkce fungovat.\n\nPřípadné dotazy a připomínky lze uskutečnit v kontaktním formuláři na webové stránce uvedené níže.\n\nInternetová stránka: http:/www.dpsc.cz\n\n© 2017 Lukáš Kolář", "Nápověda");
        }

        private void oProgramuMenuItem_Click(object sender, RoutedEventArgs e)
        {
            CustomMessageBox.Show("DATABÁZE PARKOUROVÝCH SPOTŮ V PRAZE\n\nDatabáze parkourových spotů obsahuje informace o spotech v Praze - název, fotografie, adresu, způsob dopravy na spot, velikost spotu, obtížnost skoků.\n\nFunkce:\nVyhledávání spotů podle kritérií\nŘazení spotů\nHodnocení spotů\nPřidávání spotů do oblíbených\nPřidávání vlastních soukromých spotů\nZobrazení vlastních oblíbených a ohodnocených spotů\nZbrazení vlastních spotů\n\nTato aplikace je vzájemně synchronizována s webovou stránkou http://www.dpsc.cz\n\nPřípadné dotazy a připomínky lze uskutečnit v kontaktním formuláři na webové stránce uvedené níže.\n\nInternetová stránka: http:/www.dpsc.cz\n\n© 2017 Lukáš Kolář", "O programu");
        }

        private void oblibeneMenuItem_Click(object sender, RoutedEventArgs e)
        {

            oblibeneSpotyButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
            StrankyAplikaceTabControl.SelectedItem = hlavniObrazovkaTabItem;
        }

        private void ohodnoceneMenuItem_Click(object sender, RoutedEventArgs e)
        {
            ohodnoceneSpotyButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
            StrankyAplikaceTabControl.SelectedItem = hlavniObrazovkaTabItem;
        }
        private void vlastniMenuItem_Click(object sender, RoutedEventArgs e)
        {
            vlastniSpotyButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
            StrankyAplikaceTabControl.SelectedItem = hlavniObrazovkaTabItem;
        }

        private void prihlaseniRegistraceOdhlaseniMenuItem_Click(object sender, RoutedEventArgs e)
        {
            nazevAplikaceButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }
        #endregion

        #region Hlavní stránka (hlavniObrazovkaTabItem) - Události

        // po kliknutí na tlačítko "Resetovat vyhledávání" se zavolá metoda "VymazatVyhledavani" - vymažou se filtry vyhledávání, resetuje se typ zobrazených spotů a DataGrid se znovu naplní
        private void ResetovaniVyhledavaniButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                if (TypZobrazenychSpotu != "") TypZobrazenychSpotu = "";
                VymazatVyhledavani(hlavniGrid);
                NaplneniDataGridu();
            }
        }

        // Maže uživatelem vytvořený spot
        private void smazatSpotButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                if (tabulkaSpotuDataGrid.Items.Count != 0 && (tabulkaSpotuDataGrid.SelectedItem != null || _aktualniIDSpotu != -1))
                {

                    bool result = CustomMessageBox.Show("Opravdu chcete smazat tento spot (" + ((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["Název"].ToString() + ")?", "Smazat spot", "Ano", "Ne");
                    if (result == true)
                    {
                        _sqlCe.DeleteZaznam(Convert.ToInt32(((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["ID"].ToString()), "spoty");
                        int idSpotu = Convert.ToInt32(((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["ID"].ToString());
                        string urlSpotu = ((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["Název"].ToString().CreateUrl();
                        vlastniSpotyButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                        _sqlCe.SmazaniPridruzenychZaznamuKeSpotu(idSpotu, urlSpotu);
                    }
                }
            }
            else CustomMessageBox.Show("Musíte nejdříve vybrat spot, který chcete smazat.");
        }

        // Po kliknutí na tlačítko "Zobrazit podrobnosti o spotu" se zobrazí detail spotu
        private void podrobnostiOSpotuButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                // když datagrid obsahuje záznamy a je vybraný záznam nebo je nastaveno aktuální ID spotu
                if (tabulkaSpotuDataGrid.Items.Count != 0 && (tabulkaSpotuDataGrid.SelectedItem != null || _aktualniIDSpotu != -1))
                {
                    // zobraszí se okno
                    detailSpotuTabItem.IsSelected = true;
                    polohaTabItem.IsSelected = true;

                    // nastavím aktuální ID spotu
                    if (_aktualniIDSpotu == -1) _aktualniIDSpotu = Convert.ToInt32(((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["ID"].ToString());

                    // použiji SqlCe nebo MySql List se spoty
                    SqlCeSpoty();

                    // prochází jednotlivé spoty (instance třídy Spot uložené v Listu), když se ID spotu rovná ID spotu vybraného řádku v datagridu, tak se provedou příkazy
                    foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
                    {
                        // zavolá se část kódu, která se mění podle toho, jestli je uživatel přihlášený nebo odhlášený, volá se víckrát, proto je rozdělená do samostatné metody
                        NacteniHodnoceniOblibene();

                        // nastaví se text u fotografií v závislosti na typu spotu
                        if (_typSpotu == "soukromý" || _mySqlPripojeno) fotografieInfoLabel.Content = "Pro zvětšení klikněte na fotografii.";
                        else fotografieInfoLabel.Content = "Fotografie nelze zvětšit. Zkontrolujte své internetové připojení.";

                        // nastavuje se text podle vybraného spotu
                        nazevSpotuTextBlock.Text = s.Nazev;
                        popisTextBlock.Text = s.Popis;
                        hodnoceniTextBlock.Text = s.Hodnoceni.ToString() + " / 5";
                        string adresa = (s.Adresa.Length != 0) ? ", " + s.Adresa : "";
                        adresaTextBlock.Text = s.Praha + adresa;
                        metroTextBlock.Text = s.Metro;
                        autobusTextBlock.Text = s.Bus;
                        tramvajTextBlock.Text = s.Tramvaj;
                        velikostTextBlock.Text = s.Velikost;
                        obtiznostTextBlock.Text = s.Obtiznost;
                        typTextBlock.Text = s.Typ;
                        povrchTextBlock.Text = s.Povrch;

                        // pokud spot neobsahuje určité informace, tak skryji některé viditelné prvky
                        if (s.Metro.Length != 0) leteckaMapaHyperlink.NavigateUri = new Uri(s.Mapa);
                        if (s.Streetview.Length != 0) streetViewHyperlink.NavigateUri = new Uri(s.Streetview);
                        if (s.Panorama.Length != 0) panoramaHyperlink.NavigateUri = new Uri(s.Panorama);

                        foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 1)) control.Visibility = Visibility.Visible;
                        foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 2)) control.Visibility = Visibility.Visible;
                        foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 3)) control.Visibility = Visibility.Visible;
                        if (s.Metro.Length == 0) foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 1)) control.Visibility = Visibility.Collapsed;
                        if (s.Bus.Length == 0) foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 2)) control.Visibility = Visibility.Collapsed;
                        if (s.Tramvaj.Length == 0) foreach (UIElement control in dopravaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 3)) control.Visibility = Visibility.Collapsed;
                        dopravaTabItem.Visibility = (s.Metro.Length == 0 && s.Bus.Length == 0 && s.Tramvaj.Length == 0) ? Visibility.Collapsed : Visibility.Visible;

                        foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 2)) control.Visibility = Visibility.Visible;
                        foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 3)) control.Visibility = Visibility.Visible;
                        foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 4)) control.Visibility = Visibility.Visible;
                        foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 5)) control.Visibility = Visibility.Visible;
                        if (s.Mapa.Length == 0 && s.Streetview.Length == 0 && s.Panorama.Length == 0) foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 2)) control.Visibility = Visibility.Collapsed;
                        if (s.Mapa.Length == 0) foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 3)) control.Visibility = Visibility.Collapsed;
                        if (s.Streetview.Length == 0) foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 4)) control.Visibility = Visibility.Collapsed;
                        if (s.Panorama.Length == 0) foreach (UIElement control in polohaGrid.Children.Cast<UIElement>().Where(c => Grid.GetRow(c) == 5)) control.Visibility = Visibility.Collapsed;

                        // načte se náhled fotky spotu a vytvoří se obrázky  
                        if (s.Obrazek.Length > 0)
                        {
                            string path = VytvoreniCesty(s.Obrazek);
                            Image_SourceChanged(nahledObrazkuImage, path);
                            NacteniObrazku(s);
                            fotografieTabItem.Visibility = Visibility.Visible;
                        }
                        else
                        {
                            nahledObrazkuImage.Source = new BitmapImage(new Uri("img/styl/obrazekError.png", UriKind.Relative));
                            fotografieTabItem.Visibility = Visibility.Collapsed;
                        }

                        // hledala se pouze 1 shoda v Listu -> aby se už nemuselo cyklem procházet dál zavolá se break;
                        break;
                    }

                    // vrátí se původní List z SqlCe nebo MySql
                    MySqlSpoty();
                }
                else CustomMessageBox.Show("Musíte nejdříve vybrat spot, který chcete otevřít.");
            }
        }

        // Vyhledávání v DataGridu, zavolá se metoda pro načtení dat do datagridu pro vyhledávání
        private void VyhledavaniButton_Click(object sender, EventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                if (TypZobrazenychSpotu != "") TypZobrazenychSpotu = "";
                NaplneniDataGriduVyhledavani();
            }
        }

        // řazení záznamů v DataGridu od A do Z / od Z do A
        private void RazeniAZButton_Click(object sender, RoutedEventArgs e)
        {
            // nastaví se výchozí hodnoty pro buttony řazení velikosti a hodnocení
            HodnoceniVychoziRazeni();
            VelikostVychoziRazeni();

            // zjistím směr řazení a zajistím změnu informačního textu pro řazení
            string zmena = (razeniAZButton.Content.ToString() == "Z-A") ? "A-Z" : zmena = "Z-A";
            razeniAZButtonToolTip.Text = (zmena == "A-Z") ? "Výsledky se seřadí podle názvu od A do Z" : "Výsledky se seřadí podle názvu od Z do A";
            string order = (zmena == "A-Z") ? "DESC" : "ASC";
            razeniAZButton.Content = zmena;
            razeniAZMenuItem.Header = "_" + zmena;

            // Provede se řazení datagridu
            DataView dataView = new DataView(_dataTable);
            dataView.Sort = "Název " + order;
            tabulkaSpotuDataGrid.DataContext = dataView;

            // vybere se první řádek
            if (tabulkaSpotuDataGrid.Items.Count > 0) tabulkaSpotuDataGrid.SelectedItem = tabulkaSpotuDataGrid.Items[0];
        }

        // Řazení podle části prahy spotu
        private void RazeniCastPrahyButton_Click(object sender, RoutedEventArgs e)
        {
            // nastaví se výchozí hodnoty pro buttony řazení velikosti a hodnocení a A-Z
            AZVychoziRazeni();
            HodnoceniVychoziRazeni();
            VelikostVychoziRazeni();

            // seřadí se záznamy podle části prahy
            EnumerableRowCollection<DataRow> query = from razeni in _dataTable.AsEnumerable()
                                                     orderby razeni.Field<string>("ČástPrahy").Length, razeni.Field<string>("ČástPrahy") ascending
                                                     select razeni;

            // Provede se řazení nastavením DataContextu datagridu
            tabulkaSpotuDataGrid.DataContext = query.AsDataView();

            // vybere se první řádek
            if (tabulkaSpotuDataGrid.Items.Count > 0) tabulkaSpotuDataGrid.SelectedItem = tabulkaSpotuDataGrid.Items[0];
        }


        // Řazení podle velikosti spotu
        private void RazeniVelikostSpotuButton_Click(object sender, RoutedEventArgs e)
        {
            // nastaví se výchozí hodnoty pro buttony řazení hodnocení a A-Z
            AZVychoziRazeni();
            HodnoceniVychoziRazeni();

            // zkopíruji dataTable s daty a seřadím ji abecedně a poté vymažu původní dataTable
            DataTable dataTable = new DataTable();
            dataTable = _dataTable.Copy();
            DataView dataView = new DataView(dataTable);
            dataView.Sort = "Název ASC";
            _dataTable.Rows.Clear();

            // podle směru řazení provedu řazení
            if (razeniVelikostSpotuToolTip.Text == "Výsledky se seřadí od nejmenších po největší.")
            {
                // procházím jednotlivé spoty podle jejich velikosti a vkládám je do dataTable
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Velmi malý spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Malý spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Středně velký spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Velký spot") _dataTable.ImportRow(radek.Row);

                // nastavím informační text buttonu řazení
                razeniVelikostSpotuToolTip.Text = "Výsledky se seřadí od největších po nejmenší.";
            }
            else
            {
                // procházím jednotlivé spoty podle jejich velikosti a vkládám je do dataTable
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Velký spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Středně velký spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Malý spot") _dataTable.ImportRow(radek.Row);
                foreach (DataRowView radek in dataView) if (radek["Velikost"].ToString() == "Velmi malý spot") _dataTable.ImportRow(radek.Row);

                // nastavím informační text buttonu řazení
                razeniVelikostSpotuToolTip.Text = "Výsledky se seřadí od nejmenších po největší.";
            }

            // provedu řazení
            tabulkaSpotuDataGrid.DataContext = _dataTable.AsDataView();

            // vyberu první řádek
            if (tabulkaSpotuDataGrid.Items.Count > 0) tabulkaSpotuDataGrid.SelectedItem = tabulkaSpotuDataGrid.Items[0];
        }

        // Řazení podle hodnocení
        private void RazeniHodnoceniButton_Click(object sender, RoutedEventArgs e)
        {
            // nastaví se výchozí hodnoty pro buttony řazení velikosti a A-Z
            AZVychoziRazeni();
            VelikostVychoziRazeni();

            // nastaví se informační text a uložím si typ řazení
            string sortOrder;
            if (razeniHodnoceniToolTip.Text == "Výsledky se seřadí od nejlepších po nejhorší.")
            {
                razeniHodnoceniToolTip.Text = "Výsledky se seřadí od nejhorších po nejlepší.";
                sortOrder = "DESC";
            }
            else
            {
                razeniHodnoceniToolTip.Text = "Výsledky se seřadí od nejlepších po nejhorší.";
                sortOrder = "ASC";
            }

            // seřadí se záznamy a aktualizuji datagrid
            DataView dataView = new DataView(_dataTable);
            dataView.Sort = "Hodnocení " + sortOrder;
            tabulkaSpotuDataGrid.DataContext = dataView;

            // vybere se první řádek
            if (tabulkaSpotuDataGrid.Items.Count > 0) tabulkaSpotuDataGrid.SelectedItem = tabulkaSpotuDataGrid.Items[0];
        }

        // Zobrazí se všechny spoty
        private void vsechnySpotyButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                TypZobrazenychSpotu = "";
                NaplneniDataGridu();
            }
        }

        // Zobrazí uživatelovi oblíbené spoty v datagridu
        private void OblibeneSpotyButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                TypZobrazenychSpotu = "Moje oblíbené spoty";
                NaplneniDataGriduOblibeneSpoty();
            }
        }

        // Zobrazí uživatelem ohodnocené spoty v datagridu
        private void OhodnoceneSpotyButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                TypZobrazenychSpotu = "Mnou ohodnocené spoty";
                NaplneniDataGriduHodnoceneSpoty();
            }
        }

        // Zobrazí vlastní spoty uživatele
        private void vlastniSpotyButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                TypZobrazenychSpotu = "Moje vlastní spoty";
                NaplneniDataGriduVlastniSpoty();
            }
        }

        // Otevře okno pro vyvoření vlastního spotu
        private void vytvoreniSpotuButton_Click(object sender, RoutedEventArgs e)
        {
            NovySpot ns = new NovySpot(this);
            ns.Show();
        }

        // Načte po změne výběru řádku Datagridu fotku k příslušnému řádku do Pictureboxu vedle DataGridu
        private void TabulkaSpotuDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // pokud v datagridu jsou záznamy, tak zobrazím fotku vedle datagridu, jinak ji schovám
            nahledFotkyKDataGridu.Visibility = (tabulkaSpotuDataGrid.Items.Count != 0) ? Visibility.Visible : Visibility.Collapsed;

            // pokud v datagridu jsou záznamy a pokud je záznam vybraný nebo je nastavené aktuální ID spotu
            if (tabulkaSpotuDataGrid.Items.Count != 0 && (tabulkaSpotuDataGrid.SelectedItem != null || _aktualniIDSpotu != -1))
            {
                // vyberu List se spoty
                SqlCeSpoty();

                // nastavím ID aktuálního spotu
                int id = (_aktualniIDSpotu > -1) ? _aktualniIDSpotu : Convert.ToInt32(((DataRowView)tabulkaSpotuDataGrid.SelectedItem).Row["ID"].ToString());


                // procházím spoty a vybírám spot, který se rovná aktuálnímu ID
                foreach (Spot s in _spoty.Where(s => Convert.ToInt32(s.ID.ToString()) == id && s.Obrazek != ""))
                {
                    // zobrazím / skryji button pro smazání spotu, zobrazím / nezobrazím náhled spotu a nastavím typ vybraného spotu
                    smazatSpotButton.Visibility = (s.TypSpotu == "soukromý") ? Visibility.Visible : Visibility.Collapsed;
                    nahledFotkyKDataGridu.Visibility = (s.Obrazek.Length > 0) ? Visibility.Visible : Visibility.Hidden;
                    _typSpotu = s.TypSpotu;

                    // změním fotku v datagridu         
                    string path = VytvoreniCesty(s.Obrazek); // vytvoří se cesta obrázku
                    Image_SourceChanged(nahledFotkyKDataGridu, path); // změní zdroj obrázku
                    break;
                }

                // vyberu List se spoty
                MySqlSpoty();
            }
        }

        // dvojklikem na DataGrid se otevře detail spotu podle vybraného řádku
        private async void tabulkaSpotuDataGrid_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            await Task.Delay(8);

            // vybere se řádek, když řádek není null, přejde se na detail kliknutého spotu
            DataGridRow row = ItemsControl.ContainerFromElement((DataGrid)sender, e.OriginalSource as DependencyObject) as DataGridRow;
            if (row != null) podrobnostiOSpotuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        // při načítání položek do DataGridu se ke každému řádku přidá číslo řádku
        private void tabulkaSpotuDataGrid_LoadingRow(object sender, DataGridRowEventArgs e)
        {
            e.Row.Header = (e.Row.GetIndex() + 1).ToString();
        }

        // Po stisknutí enteru v dataGridu se zobrazí detail spotu
        private void tabulkaSpotuDataGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return)))
            {
                e.Handled = true;
                podrobnostiOSpotuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
            }
        }
        #endregion

        #region Detail spotu (detailSpotuTabItem) - Události

        // po kliknutí na hvězdu (hodnocení) se zavolá funkce pro hodnocení (pokud je uživatel přihlášený)
        private void Hvezda_MouseDown(object sender, MouseButtonEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                if (IdUzivatele <= 0) CustomMessageBox.Show("Pro možnost hodnocení se musíte nejdříve přihlásit / zaregistrovat v pravém horním rohu", "Upozornění");
                else
                {
                    Image hvezda = (Image)sender;
                    int hodnoceni = int.Parse(hvezda.Name.ToString().Substring(hvezda.Name.Length - 1, 1));
                    vlastniHodnoceniStackPanel.Visibility = Visibility.Collapsed;
                    Hodnoceni(hodnoceni);
                }
            }
        }

        // Přechod na odkaz v Hyperlinku
        private void Hyperlink_RequestNavigate(object sender, RequestNavigateEventArgs e)
        {
            Process.Start(new ProcessStartInfo(e.Uri.AbsoluteUri));
            e.Handled = true;
        }

        // Kliknutí na button přejít zpět (na hlavní stranu), zobrazí se hlavní okno, vymažou se texty, obrázky z 'detailSpotuTabItem', přidá focus na dataGrid
        private void PrejitNaHlavniStranuButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                // vyberu hlavní stranu
                StrankyAplikaceTabControl.SelectedItem = hlavniObrazovkaTabItem;

                // vymažou se veškeré texty u detailu spotu
                nazevSpotuTextBlock.Text = "";
                popisTextBlock.Text = "";
                hodnoceniTextBlock.Text = "";
                adresaTextBlock.Text = "";
                metroTextBlock.Text = "";
                autobusTextBlock.Text = "";
                tramvajTextBlock.Text = "";
                velikostTextBlock.Text = "";
                obtiznostTextBlock.Text = "";
                typTextBlock.Text = "";
                povrchTextBlock.Text = "";
                tabulkaSpotuDataGrid.Focus();

                // vymažou se fotky u detailu spotu
                nahledObrazkuImage.Source = null;
                fotografieWrapPanel.Children.Clear();

                // aktuální ID spotu nastavím na -1
                _aktualniIDSpotu = -1;

                // pokud jsou vybrané v datagridu oblíbené nebo hodnocené spoty, tak je aktualizuji
                if (TypZobrazenychSpotu == "Moje oblíbené spoty") NaplneniDataGriduOblibeneSpoty();
                if (TypZobrazenychSpotu == "Mnou ohodnocené spoty") NaplneniDataGriduHodnoceneSpoty();
            }
        }

        // kliknutí na button - přidat / odebrat oblíbený spot, když je uživatel přihlášený, tak přidá do DB uživatelův oblíbený spot, jinak ho z DB vymaže
        private void OblibenyButton_Click(object sender, EventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                // pokud není uživatel přihlášený, tak se zobrazí zpráva, jinak se provede akce
                if (IdUzivatele <= 0) CustomMessageBox.Show("Pro přidání spotu do oblíbených se musíte nejdříve přihlásit / zaregistrovat v pravém horním rohu", "Upozornění");
                else
                {
                    // podle obsahu tlačítka se vybere, která akce se má provést a uloží se do proměnné 'akce'
                    string akce = "";
                    if (oblibenyButton.Content.ToString() == "Přidat do oblíbených") akce = "Přidat do oblíbených";
                    else akce = "Odebrat z oblíbených";

                    // prochází se spoty, kde ID spotu odpovídá aktuálnímu vybranému ID spotu
                    foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
                    {
                        // pokud se má spot přidat do oblíbených
                        if (akce == "Přidat do oblíbených")
                        {
                            // nastavím obsah tlačítka
                            oblibenyButton.Content = "Odebrat z oblíbených";

                            // pokud je uživatel připojený online a aktuální spot není typu soukromý
                            if (_mySqlPripojeno && _typSpotu != "soukromý")
                            {
                                // přidám oblíbený do mySql a SqlCe databáze
                                int id = _mySql.PridaniSpotuDoOblibenych(s.ID, IdUzivatele);
                                _sqlCe.PridaniSpotuDoOblibenych(s.ID, IdUzivatele, id);
                            }
                            // pokud uživatel není připojený (není online) nebo pokud je připojený a aktuální spot je typu soukromý
                            else if ((!_mySqlPripojeno) || (_mySqlPripojeno && _typSpotu == "soukromý"))
                            {
                                // přidám oblíbený pouze do SqlCe databáze
                                _sqlCe.PridaniSpotuDoOblibenych(s.ID, IdUzivatele);
                            }

                            // pokud je uživatel připojený a aktuální spot není typu soukromý, tak aktualizuji aktualizuji List se spoty z MySql databáze
                            // poté vždy aktualizuji List se spoty z SqlCe databáze
                            if (_mySqlPripojeno && _typSpotu != "soukromý") _mySql.NacteniOblibene();
                            _sqlCe.NacteniOblibene();

                            // Aktualizuji prvky na straně detailu spotu
                            NacteniHodnoceniOblibene();

                            // vypíše zprávu o úspěšné akci
                            CustomMessageBox.Show("Spot je přidaný mezi Vaše oblíbené.", "Oblíbené");
                        }
                        // pokud se má spot odebrat z oblíbených
                        else
                        {
                            // nastavím obsah tlačítka
                            oblibenyButton.Content = "Přidat do oblíbených";

                            // pokud je uživatel připojený online a aktuální spot není typu soukromý
                            if (_mySqlPripojeno && _typSpotu != "soukromý")
                            {
                                // odeberu oblíbené z obou databází
                                _mySql.OdebraniSpotuZOblibenych(s.ID, IdUzivatele);
                                _sqlCe.OdebraniSpotuZOblibenych(s.ID, IdUzivatele);
                            }
                            // pokud uživatel není připojený (není online) nebo pokud je připojený a aktuální spot je typu soukromý
                            else if ((!_mySqlPripojeno) || (_mySqlPripojeno && _typSpotu == "soukromý"))
                            {
                                // odeberu oblíbené pouze z SqlCe databáze
                                _sqlCe.PripravaOdebraniOblibene(s.ID, IdUzivatele);
                            }

                            // pokud je uživatel připojený a aktuální spot není typu soukromý, tak aktualizuji aktualizuji List se spoty z MySql databáze
                            // poté vždy aktualizuji List se spoty z SqlCe databáze
                            if (_mySqlPripojeno && _typSpotu != "soukromý") _mySql.NacteniOblibene();
                            _sqlCe.NacteniOblibene();

                            // Aktualizuji prvky na straně detailu spotu
                            NacteniHodnoceniOblibene();

                            // vypíše zprávu o úspěšné akci
                            CustomMessageBox.Show("Spot je odebraný z oblíbených.", "Oblíbené");
                        }
                        // ukončím procházení spotů
                        break;
                    }

                    // když má uživatel na hlavní straně zobrazené své oblíbené spoty -> spot se odebere z tabulky DataGridu
                    if (tabulkaSpotuDataGrid.SelectedIndex != -1) OdebraniOblibeneDatagrid(tabulkaSpotuDataGrid.SelectedIndex);
                }
            }
        }

        // když má uživatel na hlavní straně zobrazené své oblíbené spoty -> spot se odebere z tabulky DataGridu
        private void OdebraniOblibeneDatagrid(int pozice)
        {
            if (TypZobrazenychSpotu != "") _dataSet.Tables["spoty"].Rows.RemoveAt(pozice); _dataSet.AcceptChanges();
            Keyboard.ClearFocus();
        }

        // Po najetí na hvězdu se hvězdy vybarví
        private void Hvezda_MouseEnter(object sender, MouseEventArgs e)
        {
            Image hvezda = (Image)sender;
            int hodnoceni = int.Parse(hvezda.Name.ToString().Substring(hvezda.Name.Length - 1, 1));
            Image c;
            for (int i = 1; i <= 5; i++)
            {
                c = (Image)this.FindName("hvezda" + i);
                c.Source = new BitmapImage(new Uri("img/styl/hvezda-nevybrana.png", UriKind.Relative));
            }
            for (int i = 1; i <= hodnoceni; i++)
            {
                c = (Image)this.FindName("hvezda" + i);
                c.Source = new BitmapImage(new Uri("img/styl/hvezda.png", UriKind.Relative));
            }
        }

        // po odjetí z panelu pro hvězdy se hvězdám vrátí původní barva (vrátí se původní obrázek)
        private void vlastniHodnoceniStackPanel_MouseLeave(object sender, MouseEventArgs e)
        {
            Image c;
            for (int i = 1; i <= 5; i++)
            {
                c = (Image)this.FindName("hvezda" + i);
                c.Source = new BitmapImage(new Uri("img/styl/hvezda-nevybrana.png", UriKind.Relative));
            }
        }

        // Po kliknutí na načtené fotografie ke spotu se zobrazí detail fotografie
        private void Images_MouseUp(object sender, MouseButtonEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();

            // pokud se nezměnil stav připojení k internetu a zároveň je uživatel připojený k internetu nebo je typ soukromý
            if (!zmenaPripojeni && (_mySqlPripojeno || _typSpotu == "soukromý"))
            {
                // Získám obrázek a jeho rámeček a nastavím barvu rámečku
                Image c = (Image)sender;
                Border border = (Border)c.Parent;
                border.BorderBrush = new SolidColorBrush(Color.FromArgb(60, 0, 0, 0));

                // Nastavím viditelnost a text potřebným prvkům na straně galerie
                velkyObrazekNazevSpotuTextBlock.Text = nazevSpotuTextBlock.Text;
                velkyObrazekStackPanel.Opacity = 0;
                velkyObrazekNasledujiciFotoButton.Opacity = 0;
                velkyObrazekPredchoziFotoButton.Opacity = 0;
                velkyObrazekNacitaniImage.Opacity = 1;

                // zobrazím okno galerie
                galerieTabItem.IsSelected = true;

                // získám aktuální cestu k náhledu obrázku
                string path = c.Source.ToString();

                // Když typ spotu není soukromý
                if (_typSpotu != "soukromý")
                {
                    // upravím cestu k souboru, vyberu velkou verzi náhledu fotky
                    path = path.Substring(path.IndexOf("img/spoty/nahledy/"));
                    path = path.Replace("_thumb", "").Replace("img/spoty/nahledy/", "http://www.dpsc.cz/img/spoty/spoty/");
                }

                // zavolám metodu pro změnu zdroje fotografie
                Image_SourceChanged(velkyObrazekImage, path);
            }
        }

        // Mění border obrázků po najetí, odjetí, kliknutí na obrázek
        private void Images_MouseDown(object sender, MouseButtonEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();

            // pokud se nezměnil stav připojení k internetu a zároveň je uživatel připojený k internetu nebo je typ soukromý
            if (!zmenaPripojeni && (_mySqlPripojeno || _typSpotu == "soukromý"))
            {
                // změním barvu rámečku
                Image c = (Image)sender;
                Border border = (Border)c.Parent;
                border.BorderBrush = new SolidColorBrush(Color.FromArgb(180, 0, 0, 0));
            }
        }

        private void Images_MouseEnter(object sender, MouseEventArgs e)
        {
            // pokud se nezměnil stav připojení k internetu a zároveň je uživatel připojený k internetu nebo je typ soukromý
            if (_typSpotu != "soukromý")
            {
                // změním barvu rámečku
                Image c = (Image)sender;
                Border border = (Border)c.Parent;
                border.BorderBrush = new SolidColorBrush(Color.FromArgb(120, 0, 0, 0));
            }
        }

        private void Images_MouseLeave(object sender, MouseEventArgs e)
        {
            // pokud se nezměnil stav připojení k internetu a zároveň je uživatel připojený k internetu nebo je typ soukromý
            if (_typSpotu != "soukromý")
            {
                // změním barvu rámečku
                Image c = (Image)sender;
                Border border = (Border)c.Parent;
                border.BorderBrush = new SolidColorBrush(Color.FromArgb(60, 0, 0, 0));
            }
        }
        #endregion

        #region Galerie - Detail Obrázku (galerieTabItem) - Události

        // po načtení fotografie v 'galerieTabItem' se zobrazí načtená fotografie, zmizí obrázek načítání footgrafie
        private async void velkyObrazekImage_Loaded(object sender, RoutedEventArgs e)
        {
            Image c = (Image)sender;
            if (c.Source != null)
            {
                await Task.Delay(1000);
                velkyObrazekNacitaniImage.Opacity = 0;
                velkyObrazekStackPanel.Opacity = 1;
                velkyObrazekNasledujiciFotoButton.Opacity = 1;
                velkyObrazekPredchoziFotoButton.Opacity = 1;
            }
        }

        // Přejde se k předešlé fotce v galerii
        private void velkyObrazekPredchoziFotoButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                // zobrazím načítání
                velkyObrazekPredchoziFotoButton.Opacity = 0;
                velkyObrazekNasledujiciFotoButton.Opacity = 0;
                velkyObrazekStackPanel.Opacity = 0;
                velkyObrazekNacitaniImage.Opacity = 1;

                int idSpotu = 0;
                string urlSpotu = "";
                List<string> obrazky = new List<string>();

                // použiji buď SqlCe nebo MySql List se spoty
                SqlCeSpoty();

                // procházím spoty, vyberu spot, kde ID spotu se rovná aktuálnímu ID spotu
                foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
                {
                    idSpotu = s.ID; // načtu si ID stávajícího spotu
                    urlSpotu = s.Nazev.CreateUrl(); // vytvořím url z názvu spotu
                    break; // ukončím iteraci
                }

                // vrátí se původní spoty - buď SqlCe nebo MySql List se spoty
                MySqlSpoty();

                // pokud je typ spotu soukromý, tak vyberu List z obrázky z SqlCe databáze, jinak z MySql
                List<Obrazek> obrazkyList = (_typSpotu == "soukromý") ? SqlCeDatabaze.Obrazky : MySqlDatabaze.Obrazky;

                // procházím obrázky a vyberu obrázky náležející k aktuálnímu spotu
                // přidám do Listu všechny obrázky od spotu
                foreach (Obrazek o in obrazkyList) if (o.IdSpotu == idSpotu) obrazky.Add(o.Nahled);
                string[] obrazkyArr = obrazky.ToArray(); // převedu List na pole

                // z adresy stávajícího velkého obrázku vyberu pouze složku a název obrázku (slozka/nazev obrazku)
                string stavajiciObrazek = velkyObrazekImage.Source.ToString();
                stavajiciObrazek = stavajiciObrazek.Substring(stavajiciObrazek.IndexOf("/spoty/spoty/") + 13);

                // vyberu umístění exe souboru
                string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;

                int i = 0;
                string path = "";

                // procházím obrázky z Listu
                foreach (Obrazek o in obrazkyList)
                    // pokud se idSpotu z Listu rovná aktuálnámu ID spotu
                    // a zároveň stávající obrázek se rovná Fotce z listu
                    // nebo když je spot typu soukromý a zároveň náhled z listu se rovná náhledu obrázku soukromého spotu
                    if (o.IdSpotu == idSpotu && ((stavajiciObrazek == o.Fotka) || (_typSpotu == "soukromý" && urlSpotu + "/" + stavajiciObrazek.Substring(stavajiciObrazek.LastIndexOf("/") + 1) == o.Nahled)))
                    {
                        foreach (string obr in obrazkyArr)
                        {
                            if (obr == o.Nahled)
                            {
                                // když je pořadí předešlého obrázk menší než 0 
                                if (i - 1 < 0) path = obrazkyArr[obrazkyArr.Length - 1]; // tak vyberu poslední fotku aktuálního spotu
                                else path = obrazkyArr[i - 1]; // jinak vyberu předcházející fotku

                                // když se nejedná o soukromý spot
                                if (_typSpotu != "soukromý")
                                {
                                    // vytvořím cestu k velkému brázku pomocí webové stránky
                                    path = "http://www.dpsc.cz/img/spoty/spoty/" + path;
                                    path = path.Replace("_thumb", "");
                                }
                                else
                                {
                                    // vytvořím cestu ručně z místního souboru
                                    path = baseDirectory + "img\\spoty\\nahledy\\" + path.Replace("/", "\\");
                                }
                                break;
                            }
                            i++;
                        }
                        // zavolám metodu pro změnu zdroje obrázku
                        Image_SourceChanged(velkyObrazekImage, path);

                    }
            }
        }
        // Přejde se k následující fotce v galerii
        private void velkyObrazekNasledujiciFotoButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                // zobrazím načítání
                velkyObrazekPredchoziFotoButton.Opacity = 0;
                velkyObrazekNasledujiciFotoButton.Opacity = 0;
                velkyObrazekStackPanel.Opacity = 0;
                velkyObrazekNacitaniImage.Opacity = 1;

                int idSpotu = 0;
                string urlSpotu = "";
                List<string> obrazky = new List<string>();

                // použiji buď SqlCe nebo MySql List se spoty
                SqlCeSpoty();

                // procházím spoty, vyberu spot, kde ID spotu se rovná aktuálnímu ID spotu
                foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
                {
                    idSpotu = s.ID; // načtu si ID stávajícího spotu
                    urlSpotu = s.Nazev.CreateUrl(); // vytvořím url z názvu spotu
                    break; // ukončím iteraci
                }

                // vrátí se původní spoty - buď SqlCe nebo MySql List se spoty
                MySqlSpoty();

                // pokud je typ spotu soukromý, tak vyberu List z obrázky z SqlCe databáze, jinak z MySql
                List<Obrazek> obrazkyList = (_typSpotu == "soukromý") ? SqlCeDatabaze.Obrazky : MySqlDatabaze.Obrazky;

                // procházím obrázky a vyberu obrázky náležející k aktuálnímu spotu
                // přidám do Listu všechny obrázky od spotu
                foreach (Obrazek o in obrazkyList) if (o.IdSpotu == idSpotu) obrazky.Add(o.Nahled);
                string[] obrazkyArr = obrazky.ToArray(); // převedu List na pole

                // z adresy stávajícího velkého obrázku vyberu pouze složku a název obrázku (slozka/nazev obrazku)
                string stavajiciObrazek = velkyObrazekImage.Source.ToString();
                stavajiciObrazek = stavajiciObrazek.Substring(stavajiciObrazek.IndexOf("/spoty/spoty/") + 13);

                // vyberu umístění exe souboru
                string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;

                int i = 0;
                string path = "";

                // Z listu vyberu obrázky se správným ID spotu podle dříve získaného ID spotu stávajícího obrázku ->
                // když stávající obrázek se rovná obrázku z třídy 'Obrazek', tak potom kontroluji jestli je index i + 1 (což se rovná ->
                // -> pořadí následujícího obrázku z třídy 'Obrazek') > než délka pole obrázků, když ano, tak vyberu z pole obrázků první obrázek, jinak -> 
                // vyberu obrázek z pole s indexem o 1 větší než je index stávajícího obrázku -> poté sestavím finální adresu obrázku a načtu obrázek

                // procházím obrázky z Listu
                foreach (Obrazek o in obrazkyList)
                    // pokud se idSpotu z Listu rovná aktuálnámu ID spotu
                    // a zároveň stávající obrázek se rovná Fotce z listu
                    // nebo když je spot typu soukromý a zároveň náhled z listu se rovná náhledu obrázku soukromého spotu
                    if (o.IdSpotu == idSpotu && ((stavajiciObrazek == o.Fotka) || (_typSpotu == "soukromý" && urlSpotu + "/" + stavajiciObrazek.Substring(stavajiciObrazek.LastIndexOf("/") + 1) == o.Nahled)))
                    {
                        foreach (string obr in obrazkyArr)
                        {
                            if (obr == o.Nahled)
                            {
                                // když je pořadí předešlého obrázku menší než 0 
                                if (i + 1 >= obrazkyArr.Length) path = obrazkyArr[0]; // tak vyberu první fotku aktuálního spotu
                                else path = obrazkyArr[i + 1]; // jinak vyberu následující fotku

                                // když se nejedná o soukromý spot
                                if (_typSpotu != "soukromý")
                                {
                                    // vytvořím cestu k velkému brázku pomocí webové stránky
                                    path = "http://www.dpsc.cz/img/spoty/spoty/" + path;
                                    path = path.Replace("_thumb", "");
                                }
                                else
                                {
                                    // vytvořím cestu ručně z místního souboru
                                    path = baseDirectory + "img\\spoty\\nahledy\\" + path.Replace("/", "\\");
                                }
                                break;
                            }
                            i++;
                        }
                        // zavolám metodu pro změnu zdroje obrázku
                        Image_SourceChanged(velkyObrazekImage, path);

                    }
            }
        }

        // Přejde se zpět k detailu spotu a vymaže se zdroj velkého obrázku
        private void VelkyObrazekPrejitZpetButton_Click(object sender, RoutedEventArgs e)
        {
            velkyObrazekImage.Source = null;
            StrankyAplikaceTabControl.SelectedItem = detailSpotuTabItem;
        }
        #endregion

        #region Metody volané z událostí

        // Metody zajišťující výchozí řazení buttonů
        private void AZVychoziRazeni()
        {
            razeniAZButton.Content = "A-Z";
            razeniAZButtonToolTip.Text = "Výsledky se seřadí podle názvu od A do Z";
            razeniAZMenuItem.Header = "_A-Z";
        }
        private void HodnoceniVychoziRazeni()
        {
            razeniHodnoceniToolTip.Text = "Výsledky se seřadí od nejlepších po nejhorší.";
        }
        private void VelikostVychoziRazeni()
        {
            razeniVelikostSpotuToolTip.Text = "Výsledky se seřadí od nejmenších po největší.";
        }

        // Vytváří se cesta k náhledům spotu
        private string VytvoreniCesty(string obrazek)
        {
            string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
            string slozka = obrazek.Substring(0, obrazek.LastIndexOf("/"));
            slozka = baseDirectory + "img\\spoty\\nahledy\\" + slozka;
            string soubor = obrazek.Substring(obrazek.LastIndexOf("/") + 1);
            slozka = slozka + "\\" + soubor;
            return slozka;
        }

        // připojuje se k online databázi
        private void MySqlPripojovani()
        {
            try
            {
                _mySql = new MySqlDatabaze();
                _mySqlPripojeno = true;
            }
            catch
            {
                _mySqlPripojeno = false;
            }
        }

        // připojuje se k offline databázi
        private void SqlCePripojovani()
        {
            try
            {
                _sqlCe = new SqlCeDatabaze();
                _sqlCePripojeno = true;
            }
            catch
            {
                _sqlCePripojeno = false;
            }
        }

        // nastavují se detaily pro dokončení připojení k databázi
        private void DokonceniDBPripojeni()
        {
            // pokud je aplikace připojená k internetu, tak přizpůsobým vzhled aplikace a jako používaný list zvolím MySql list spotů
            if (_mySqlPripojeno)
            {
                nazevAplikaceTextBlock.Text = "Databáze parkourových spotů v Praze";
                nazevAplikacePripojeniButton.Visibility = Visibility.Collapsed;
                _spoty = MySqlDatabaze.Spoty;
            }
            // jinak přizpůsobým vzhled jinak a spoty vyberu z SqlCe databáze
            else
            {
                nazevAplikaceTextBlock.Text = "Databáze parkourových spotů v Praze (offline)";
                nazevAplikacePripojeniButton.Visibility = Visibility.Visible;
                _spoty = SqlCeDatabaze.Spoty;
            }

            // pokud není zobrazena hlavní strana, tak vynutím kliknutí na detail spotu 
            if (detailSpotuTabItem.IsSelected || galerieTabItem.IsSelected) podrobnostiOSpotuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        // Metoda vypíše chybové hlášky, když se nejde připojit k některé z databází
        private void ChyboveHlasky()
        {
            // Nastavím časovač a jeho interval a spustím ho
            DispatcherTimer timer = new DispatcherTimer();
            int timeout = -1;
            timer.Interval = TimeSpan.FromMilliseconds(500);
            timeout = 1;
            timer.Start();
            timer.Tick += (s, e) =>
            {
                // pokud je již zobrazena hlavní okno aplikace
                if (this.Visibility == Visibility.Visible)
                {
                    timeout--;
                    if (timeout == 0) // pokud je časovač na nule
                    {
                        // pokud se nelze připojit k online (MySql) databázi, vypíšu zprávu
                        if (!_mySqlPripojeno && _sqlCePripojeno)
                        {
                            CustomMessageBox.Show("Nelze se připojit k online databázi.\nNěkteré funkce nebudou přístupné.\nZkontrolujte internetové připojení, znovu připojit se lze kliknutím na tlačítko - 'Připojit se k online databázi' v pravém horním rohu.\n\nDěkujeme.", "Chyba připojení");
                        }
                        // pokud se nelze připojit k offline (SqlCe) databázi a ani k MySql databázi, vypíšu zprávu a zastavím funkčnost hlavní obrazovky
                        else if (!_mySqlPripojeno && !_sqlCePripojeno)
                        {
                            CustomMessageBox.Show("Nelze se připojit k online ani k offline (místní) databázi.\n\nZkuste aplikaci zavřít a znovu otevřít.\nDěkujeme.", "Chyba připojení");
                            UvodniObrazovka.IsEnabled = false;
                        }

                        // zastavím časovač
                        timer.Stop();
                    }
                }
            };
        }

        private bool _zobrazitAplikaci = false;
        // Metoda, která se snaží připojit k databázím
        private void PripojeniKDatabazi()
        {
            // spustím BackgroundWorker, pracování na pozadí
            BackgroundWorker bw = new BackgroundWorker();
            bw.DoWork += (s, e) =>
            {
                // pokud je internet přístupný
                if (JePristupnyInternet())
                {
                    _splashScreen.NacitaniText = "Načítání online databáze"; // zobrazím zprávu o připojování
                    MySqlPripojovani(); // snažím se připojit k MySql databázi
                }
                // pokud není připojení k interentu přístupné
                else
                {
                    _mySqlPripojeno = false;
                }
                _splashScreen.NacitaniText = "Načítání offline databáze";  // zobrazím zprávu o připojování
                SqlCePripojovani();  // snažím se připojit k SqlCe databázi

                // pokud bylo připojení k MySql a SqlCe databázi úspěšné
                if (_mySqlPripojeno && _sqlCePripojeno)
                {
                    _splashScreen.NacitaniText = "Synchronizace dat"; // zobrazím zprávu o synchronizaci dat

                    // začnu synchronizovat data mezi oběma databázema
                    _mySql.SynchronizaceDat(_sqlCe);
                    if (!_sqlCe.ExistujiNahledy()) _splashScreen.NacitaniText = "Vytváří se náhledy fotografií. To může trvat i několik minut."; // zobrazím zprávu o vytváření náhledů
                    _sqlCe.SynchronizaceDat(_mySql);
                }
                _zobrazitAplikaci = true;
            };
            // Background Worker se spustí a začne pracovat asynchronně
            bw.RunWorkerAsync();

            // spustím Časovač a nastavím jeho interval a zapnu ho
            DispatcherTimer timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(500);
            timer.Start();
            timer.Tick += (s, e) =>
            {
                // pokud se již má zobrazit aplikace = vše se načetlo
                if (_zobrazitAplikaci)
                {
                    // Zobrazím  zprávu o dokončování
                    _splashScreen.NacitaniText = "Načítání dat a dokončování";

                    // Naplním datagrid
                    NaplneniDataGridu();

                    // Zavolá se metoda pro dokončení připojení (změny v prvcích aplikace pro online / offline připojení)
                    DokonceniDBPripojeni();

                    timer.Stop(); // zastavím časovač
                    _splashClose = true; // nastavím proměnnou splashClose na true - má se zavřít Splash Screen
                    ChyboveHlasky(); // Zavolám metodu s chybovými hláškami
                }
            };
        }

        // Metoda zjišťuje zda je k dispozici internetové připojení
        private bool JePristupnyInternet()
        {
            return System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable();
        }

        // Kontroluje internetové připojení, na základě toho, jestli je uživatel připojený k internetu buď zavolá metody pro připojení k online nebo k offline databázi
        public bool ZmenaPripojeniInternetu()
        {
            bool jePristupnyInternet = JePristupnyInternet();
            BackgroundWorker bw = new BackgroundWorker();
            bool mySqlOk = false;
            mySqlOk = _mySqlPripojeno;

            // aplikace se snaží připojit k online databázi
            if (_mySqlPripojeno && !jePristupnyInternet) // když je nastaveno, že funguje online databáze
            {
                oznameniTextBlock.Text = "Online databáze není k dispozici. Bude použita offline databáze.\nOffline databáze se načítá.";
                bw.DoWork += (s, e) =>
                {
                    SqlCePripojovani(); // aplikace se snaží připojit k místní databázi
                    _mySqlPripojeno = false; // nastaví se proměnná kontrolující online připojení na false
                };
            }

            else
            if (!_mySqlPripojeno && jePristupnyInternet) // když je nastaveno, že funguje online databáze
            {
                MySqlPripojovani();
                if ((!mySqlOk && !_mySqlPripojeno && jePristupnyInternet))
                {
                    return false;
                }
                // nastaví se zpráva
                oznameniTextBlock.Text = "Online databáze je k dispozici. Bude použita online databáze.\nOnline databáze se načítá.";
                bw.DoWork += (s, e) =>
                {
                    if (_mySqlPripojeno && _sqlCePripojeno) // když fungují obě databáze
                    {
                        // zesynchronizuji data
                        _mySql.SynchronizaceDat(_sqlCe);
                        _sqlCe.SynchronizaceDat(_mySql);
                    }
                };
            }
            // pokud došlo ke změně připojení
            if ((mySqlOk && !jePristupnyInternet) || (!mySqlOk && jePristupnyInternet))
            {
                bw.RunWorkerAsync(); // spustí Background Worker
                if (galerieTabItem.IsSelected) detailSpotuTabItem.IsSelected = true;
                oznameniGrid.Visibility = Visibility.Visible; // zobrazí zprávu

                // Nastaví se a spustí se časovač
                DispatcherTimer timer = new DispatcherTimer();
                int timeout = 8;
                timer.Interval = TimeSpan.FromMilliseconds(500);
                timer.Start();
                timer.Tick += (s, e) =>
                {
                    if (timeout > 0) timeout--;
                    // pokud vypršel časovač a aplikace je připojená alespoň k jedné databázi
                    if ((_mySqlPripojeno || _sqlCePripojeno) && timeout == 0)
                    {
                        NaplneniDataGridu(); // naplní se datagrid
                        oznameniGrid.Visibility = Visibility.Collapsed; // skryje se zpráva
                        DokonceniDBPripojeni(); // nastavují se detaily pro dokončení připojení k databázi

                        timer.Stop(); // ukončím časovač

                        // vypíše hlášky o připojení
                        if (_mySqlPripojeno) CustomMessageBox.Show("Nyní jste úspěšně připojeni k online databázi.\n\nDěkujeme.", "Připojení");
                        if (_sqlCePripojeno && !_mySqlPripojeno) CustomMessageBox.Show("Nyní jste úspěšně připojeni k místní (offline) databázi.\nZkontrolujte internetové připojení, znovu připojit se lze kliknutím na tlačítko - 'Připojit se k online databázi' v pravém horním rohu\n\nDěkujeme.", "Připojení");
                    }
                };
                return true; // vrátí se true, tzn. byla provedena změna připojení
            }
            else
            {
                return false; // vrátí se false, tzn. nebyla provedena změna připojení
            }
        }

        // Vybírá výsledky vyhledávání
        private void NaplneniDataGriduVyhledavani()
        {
            // vytvoří se příkaz pro vybrání hledaných spotů
            _query = (_mySqlPripojeno) ? _mySql.VybraniVysledku(nazevSpotuTextBox.Text, vyhledavaniCastPrahyComboBox.Text, vyhledavaniPovrchComboBox.Text, vyhledavaniObtiznostComboBox.Text, vyhledavaniVelikostSpotuComboBox.Text, vyhledavaniTreninkComboBox.Text) : _sqlCe.VybraniVysledku(nazevSpotuTextBox.Text, vyhledavaniCastPrahyComboBox.Text, vyhledavaniPovrchComboBox.Text, vyhledavaniObtiznostComboBox.Text, vyhledavaniVelikostSpotuComboBox.Text, vyhledavaniTreninkComboBox.Text, IdUzivatele);

            // Záznamy se načtou do DataGridu
            NacteniZaznamu();

            // pokud je uživatel připojený k online databázi a uživatel je přihlášený
            if (_mySqlPripojeno && IdUzivatele > 0)
            {
                // vytvoří se příkaz pro vybrání hledaných soukromých spotů
                _query = _sqlCe.VybraniVysledku(nazevSpotuTextBox.Text, vyhledavaniCastPrahyComboBox.Text, vyhledavaniPovrchComboBox.Text, vyhledavaniObtiznostComboBox.Text, vyhledavaniVelikostSpotuComboBox.Text, vyhledavaniTreninkComboBox.Text, IdUzivatele);

                // Načtou se soukromé záznamy do DataGridu
                NacteniSoukromychZaznamu();
            }

            // Pokud hledání neobsahuje žádné výsledky, vypíše se zpráva
            if (tabulkaSpotuDataGrid.Items.Count == 0) CustomMessageBox.Show("Žádné spoty podle Vašich kritérií nejsou k dispozici", "Oznámení");

            // Vybere se řádek v datagridu
            VybraniRadkuDatagridu();
        }

        // Vybírá všechny uživatelem hodnocené spoty
        private void NaplneniDataGriduHodnoceneSpoty()
        {
            // vytvoří se příkaz pro vybrání uživatelem ohodnocených spotů
            _query = (_mySqlPripojeno) ? _mySql.VybraniVysledku(IdUzivatele, "hodnocení") : _sqlCe.VybraniVysledku(IdUzivatele, "hodnocení");

            // Záznamy se načtou do DataGridu
            NacteniZaznamu();

            // pokud je uživatel připojený k online databázi a uživatel je přihlášený
            if (_mySqlPripojeno && IdUzivatele > 0)
            {
                _query = _sqlCe.VybraniVysledku(IdUzivatele, "hodnocení"); // vytvoří se příkaz pro vybrání uživatelem ohodnocených soukromých spotů
                NacteniSoukromychZaznamu(); // Načtou se soukromé záznamy do DataGridu
            }

            // Pokud hledání neobsahuje žádné výsledky, vypíše se zpráva
            if (tabulkaSpotuDataGrid.Items.Count == 0) CustomMessageBox.Show("Zatím nemáte ohodnocené žádné spoty", "Oznámení");

            // Vybere se řádek v datagridu
            VybraniRadkuDatagridu();
        }

        private void NaplneniDataGriduOblibeneSpoty()
        {
            // vytvoří se příkaz pro vybrání uživatelových oblíbených spotů
            _query = (_mySqlPripojeno) ? _mySql.VybraniVysledku(IdUzivatele, "oblíbené") : _sqlCe.VybraniVysledku(IdUzivatele, "oblíbené");

            // Záznamy se načtou do DataGridu
            NacteniZaznamu();

            // pokud je uživatel připojený k online databázi a uživatel je přihlášený
            if (_mySqlPripojeno && IdUzivatele > 0)
            {
                _query = _sqlCe.VybraniVysledku(IdUzivatele, "oblíbené"); // vytvoří se příkaz pro vybrání uživatelových oblíbených soukromých spotů
                NacteniSoukromychZaznamu(); // Načtou se soukromé záznamy do DataGridu
            }

            // Pokud hledání neobsahuje žádné výsledky, vypíše se zpráva
            if (tabulkaSpotuDataGrid.Items.Count == 0) CustomMessageBox.Show("Zatím nemáte žádné spoty v oblíbených", "Oznámení");

            // Vybere se řádek v datagridu
            VybraniRadkuDatagridu();
        }

        public void NaplneniDataGriduVlastniSpoty()
        {
            // vytvoří se příkaz pro vybrání uživatelem vytvořených spotů
            _query = _sqlCe.VybraniVysledku(IdUzivatele, "vlastní");

            // Záznamy se načtou do DataGridu
            NacteniZaznamu();

            // Pokud hledání neobsahuje žádné výsledky, vypíše se zpráva
            if (tabulkaSpotuDataGrid.Items.Count == 0) CustomMessageBox.Show("Zatím nemáte vytvořené žádné vlastní spoty.", "Oznámení");

            // Vybere se řádek v datagridu
            VybraniRadkuDatagridu();
        }



        private void NaplneniDataGridu()
        {
            // vytvoří se příkaz pro vybrání všech veřejných spotů
            _query = (_mySqlPripojeno) ? _mySql.VybraniVysledku() : _sqlCe.VybraniVysledku(IdUzivatele);

            // Záznamy se načtou do DataGridu
            NacteniZaznamu();

            // pokud je uživatel připojený k online databázi a uživatel je přihlášený
            if (_mySqlPripojeno && IdUzivatele > 0)
            {
                _query = _sqlCe.VybraniVysledku(IdUzivatele); // vytvoří se příkaz pro vybrání všech uživatelových soukromých spotů
                NacteniSoukromychZaznamu(); // Načtou se soukromé záznamy do DataGridu
            }

            // Vybere se řádek v datagridu
            VybraniRadkuDatagridu();
        }

        // Načítá všechny spoty a přiřazuje je do datagridu
        private void NacteniZaznamu()
        {
            _dataSet = (_mySqlPripojeno && TypZobrazenychSpotu != "Moje vlastní spoty") ? _mySql.NacteniZaznamu(_query) : _sqlCe.NacteniZaznamu(_query);
            _dataTable = _dataSet.Tables["spoty"];
            tabulkaSpotuDataGrid.DataContext = _dataSet.Tables["spoty"].DefaultView;
        }

        // Načítá soukromé spoty a přiřazuje je do datagridu
        private void NacteniSoukromychZaznamu()
        {
            // změním příkaz pro vybrání všech soukromých spotů
            _query = (_query.Contains("typ_spotu = 'veřejný' OR s.id_uzivatele")) ? _query.Replace("typ_spotu = 'veřejný' OR s.id_uzivatele", "typ_spotu = 'soukromý' AND s.id_uzivatele") : _query;

            // načtu soukromé spoty do DataSetu
            DataSet soukromeSpotyDataSet = _sqlCe.NacteniSoukromychSpotuVyhledavani(_query, IdUzivatele);

            // Spojím veřejné a soukromé spoty
            _dataTable.Merge(soukromeSpotyDataSet.Tables["soukrome_spoty"], true);

            // seřadím spoty od A-Z
            AZVychoziRazeni();
            razeniAZButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
        }

        // Vybírá první řádek v DataGridu
        private async void VybraniRadkuDatagridu()
        {
            await Task.Delay(10);
            if (tabulkaSpotuDataGrid.Items.Count > 0) tabulkaSpotuDataGrid.SelectedItem = tabulkaSpotuDataGrid.Items[0];
        }

        // Pokud je uživatel připojený k online (MySql) databázi a je přihlášený, tak se použijí Spoty z listu SqlCe databáze
        private void SqlCeSpoty()
        {
            _spoty = (_mySqlPripojeno && IdUzivatele > 0) ? SqlCeDatabaze.Spoty : _spoty;
        }

        // Pokud je uživatel připojený k online (MySql) databázi a je přihlášený, tak se použijí Spoty z listu MySql databáze
        private void MySqlSpoty()
        {
            _spoty = (_mySqlPripojeno && IdUzivatele > 0) ? MySqlDatabaze.Spoty : _spoty;
        }


        // Zobrazí možnosti pro přihlášené uživatele
        private void DokonceniPrihlaseni()
        {
            // Zobrazí uživatelovo jméno do horní části aplikace
            nazevAplikaceUzivatelTextBlock.Text = Uzivatel;

            // Změní text tlačítko a tím i akci, která se provede po kliknutí
            nazevAplikaceButton.Content = "Odhlásit se";

            // Zviditelní se uživatelovo jméno v horní části aplikace
            nazevAplikaceUzivatelTextBlock.Visibility = Visibility.Visible;

            // Zviditelní se, pro vybrání, typy spotů na hlavní straně
            prihlaseniButtonyStackPanel.Visibility = Visibility.Visible;

            // Skryjí / zviditelní se položky v menu
            prihlaseniMenuItem.Visibility = Visibility.Collapsed;
            registraceMenuItem.Visibility = Visibility.Collapsed;
            odhlaseniMenuItem.Visibility = Visibility.Visible;
            mojeSpotyMenuItem.Visibility = Visibility.Visible;
        }

        // Skryje možnosti pro přihlášené uživatele
        private void DokonceniOdhlaseni()
        {
            // Zobrazí uživatelovo jméno do horní části aplikace
            nazevAplikaceUzivatelTextBlock.Text = Uzivatel;

            // Změní text tlačítko a tím i akci, která se provede po kliknutí
            nazevAplikaceButton.Content = "Přihlásit se / Registrovat se";

            // Skryje se uživatelovo jméno v horní části aplikace
            nazevAplikaceUzivatelTextBlock.Visibility = Visibility.Collapsed;

            // Skryjí se typy spotů na hlavní straně
            prihlaseniButtonyStackPanel.Visibility = Visibility.Collapsed;

            // Skryjí / zviditelní se položky v menu
            prihlaseniMenuItem.Visibility = Visibility.Visible;
            registraceMenuItem.Visibility = Visibility.Visible;
            odhlaseniMenuItem.Visibility = Visibility.Collapsed;
            mojeSpotyMenuItem.Visibility = Visibility.Collapsed;

            // Pokud nejsou zobrazeny všechny spoty, tak se nastaví typ zobrazených spotů
            if (TypZobrazenychSpotu != "") TypZobrazenychSpotu = "";

            // pokud uživatel není na hlavní straně
            if (detailSpotuTabItem.IsSelected || galerieTabItem.IsSelected)
            {

                // pokud je aktuální spot typu soukromý, tak se přejde na hlavní stranu
                if (_typSpotu == "soukromý") prejitNaHlavniStranuButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
            }
        }

        // po přihlášení / odhlášení uživatele se skryje / zobrazí uživatelovo hodnocení spotu a oblíbené
        private void NacteniHodnoceniOblibene()
        {
            // když aktuální ID spotu se nerovná -1, tzn. jsem na straně detailu spotu nebo galerie
            if (_aktualniIDSpotu != -1)
            {

                // aktualizují se spoty
                if (_mySqlPripojeno) _mySql.NacteniSpotu();
                _sqlCe.NacteniSpotu();
                SqlCeSpoty();

                // procházím spoty a hledám spot s ID aktuálního spotu
                foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
                {
                    // pokud je uživatel připojený k online databázi nebo je přihlášený
                    if (_mySqlPripojeno || IdUzivatele > 0)
                    {
                        string hodnoceni = "";
                        int index = -1;

                        // pokud je uživatel připojený k online databázi a aktuální spot není soukromý spot,
                        // tak se načtou oblíbené a hodnocení do MySql Listů a načte se uživatelovo hodnocení z MySql
                        if (_mySqlPripojeno && _typSpotu != "soukromý")
                        {
                            _mySql.NacteniHodnoceni();
                            _mySql.NacteniOblibene();
                            hodnoceni = _mySql.ZiskatHodnoceni(IdUzivatele, s.ID);

                            // hledá se v online databázi, jestli je aktuální spot oblíbený spot uživatele
                            index = MySqlDatabaze.Oblibene.FindIndex(x => x.IdUzivatele == IdUzivatele && x.IdSpotu == s.ID);
                        }

                        // pokud není uživatel připojený k online databázi a je přihlášený
                        // nebo když je připojený k online databázi a aktuální spot je soukromý
                        // tak se načtou oblíbené a hodnocení do SqlCe Listů a načte se uživatelovo hodnocení z SqlCe
                        else if ((!_mySqlPripojeno && IdUzivatele > 0) || (_mySqlPripojeno && _typSpotu == "soukromý"))
                        {
                            _sqlCe.NacteniHodnoceni();
                            _sqlCe.NacteniOblibene();
                            hodnoceni = _sqlCe.ZiskatHodnoceni(IdUzivatele, s.ID);

                            // hledá se v offline databázi, jestli je aktuální spot oblíbený spot uživatele
                            index = SqlCeDatabaze.Oblibene.FindIndex(x => x.IdUzivatele == IdUzivatele && x.IdSpotu == s.ID && x.SqlCe != 0);
                        }

                        // když uživatel u aktuálního spotu má vlastní hodnocením tak se skryje možnost spot hodnotit
                        // a zobrazí se jeho hodnocení
                        if (hodnoceni != "")
                        {
                            vlastniHodnoceniTextBlock.Text = "Vaše hodnocení: " + hodnoceni + " / 5";
                            vlastniHodnoceniStackPanel.Visibility = Visibility.Collapsed;
                        }
                        // když nemá vlastní hodnocení, tak se zobrazí možnost hodnotit spot
                        else
                        {
                            vlastniHodnoceniTextBlock.Text = "Vaše hodnocení:";
                            vlastniHodnoceniStackPanel.Visibility = Visibility.Visible;
                        }

                        // pokud je aktuální spot již uživatelem přidaný do oblíbených, zobrazím to
                        if (index >= 0)
                        {
                            oblibenyImageStackPanel.Visibility = Visibility.Visible;
                            oblibenyButton.Content = "Odebrat z oblíbených";
                        }
                        // pokud aktuální spot není uživatelem přidaný do oblíbených, zobrazím to
                        else
                        {
                            oblibenyButton.Content = "Přidat do oblíbených";
                            oblibenyImageStackPanel.Visibility = Visibility.Collapsed;
                        }
                    }
                    // pokud uživatel není přihlášený přizpůsobím stránku detailu spotu
                    else
                    {
                        vlastniHodnoceniTextBlock.Text = "Vaše hodnocení:";
                        oblibenyImageStackPanel.Visibility = Visibility.Collapsed;
                        vlastniHodnoceniStackPanel.Visibility = Visibility.Visible;
                    }
                    break;
                }
                MySqlSpoty();
            }
        }

        // Načítá a mění zdroj obrázku
        private void Image_SourceChanged(Image sender, string path)
        {
            Image c = (Image)sender;
            BitmapImage image = new BitmapImage();
            bool nacteno = false;

            // procházím již načtené fotky spotů
            foreach (BitmapImage i in _nacteneObrazky)
            {
                // pokud zadaná cesta k obrýzku je již v Listu načtených obrázků
                if (i.UriSource.ToString() == @path)
                {
                    // pokud se jedná o velký obrázek v galerii, zobrazím ho namísto načítání
                    if (c.Name == velkyObrazekImage.Name)
                    {
                        velkyObrazekStackPanel.Opacity = 1;
                        velkyObrazekNasledujiciFotoButton.Opacity = 1;
                        velkyObrazekPredchoziFotoButton.Opacity = 1;
                        velkyObrazekNacitaniImage.Opacity = 0;
                        nacteniFotografieProgressBar.Value = 0;
                    }

                    nacteno = true;
                    c.SetCurrentValue(Image.SourceProperty, i); // změním zdroj obrázku
                    break;
                }
            }

            // pokud stávající obrázek není v Listu již načtených obrázků
            if (!nacteno)
            {
                // vytvořím obrázek pomocí cesty zadané v parametru metody
                image.BeginInit();
                image.CacheOption = BitmapCacheOption.OnLoad;
                image.UriSource = new Uri(@path);
                image.EndInit();

                // při nepovedeném stažení obrázku nastavím chybový obrázek
                image.DownloadFailed += (s, e) =>
                {
                    c.Source = new BitmapImage(new Uri("img/styl/obrazekError.png", UriKind.Relative));
                };

                c.SetCurrentValue(Image.SourceProperty, image); // změním zdroj obrázku
                _nacteneObrazky.Add(image); // přidám obrázek do načtených obrázků

                // pokud se jedná o obrázek z webové stránky, tak načítám progressbar a po stáhnutí načtu fotku
                if (c.Name == velkyObrazekImage.Name && !path.Contains("img\\spoty\\nahledy\\"))
                {
                    image.DownloadProgress += (s, e) => nacteniFotografieProgressBar.Value = e.Progress;
                    image.DownloadCompleted += (s, e) =>
                    {
                        ZobrazeniFotky();
                    };
                }
                // jinak načtu fotku ihned
                else
                {
                    ZobrazeniFotky();
                }
            }
        }

        // Skryje načítání fotky, vynuluje progressbar a zobrazí načtenou fotografii
        private void ZobrazeniFotky()
        {
            velkyObrazekStackPanel.Opacity = 1;
            velkyObrazekNasledujiciFotoButton.Opacity = 1;
            velkyObrazekPredchoziFotoButton.Opacity = 1;
            velkyObrazekNacitaniImage.Opacity = 0;
            nacteniFotografieProgressBar.Value = 0;
        }

        // slouží pro vytvoření a načtení obrázků k vybranému spotu do okna detailu spotu
        private void NacteniObrazku(Spot s)
        {
            int i = 0;
            string path = "";

            // procházím obrázky a vybírám takové, kde ID spotu se rovná ID aktuálního vybraného spotu
            foreach (Obrazek o in SqlCeDatabaze.Obrazky.Where(x => x.IdSpotu == s.ID))
            {
                i++; // zvětším počítání o 1čku

                // nastavím styly rámečku obrázku
                Border b = new Border();
                b.Margin = new Thickness(10);
                b.HorizontalAlignment = HorizontalAlignment.Left;
                b.VerticalAlignment = VerticalAlignment.Top;
                b.BorderThickness = new Thickness(2);
                b.BorderBrush = new SolidColorBrush(Color.FromArgb(60, 0, 0, 0));

                b.Name = "obrazkyPBBorder" + (i - 1).ToString(); // nastavím jméno rámečku obrázku
                fotografieWrapPanel.Children.Add(b); // přidám rámeček obrázku do WrapPanelu

                // vytvořím obrázek, natavím mu jméno, vytvořím cestu, změním zdroj, nastavím styly obrázku
                // přiřadím obrázek do rámečku a přiřadím události obrázku
                Image c = new Image();
                c.Name = "obrazkyPB" + (i - 1).ToString();
                path = VytvoreniCesty(o.Nahled);
                Image_SourceChanged(c, path);
                c.Width = 350;
                c.Height = 263;
                c.HorizontalAlignment = HorizontalAlignment.Left;
                c.VerticalAlignment = VerticalAlignment.Top;
                b.Child = c;
                c.MouseDown += Images_MouseDown;
                c.MouseUp += Images_MouseUp;
                c.MouseEnter += Images_MouseEnter;
                c.MouseLeave += Images_MouseLeave;
            }
        }

        // Metoda přidává uživatelovo hodnocení ke spotu
        private void Hodnoceni(int hodnoceni)
        {
            // procházím spoty, kde ID aktuálního spotu se rovná ID spotu v Listu
            foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
            {
                // pokud je uživatel připojený k online databázi a hodnotí veřejný spot
                if (_mySqlPripojeno && _typSpotu != "soukromý")
                {
                    // přidám hodnocení do obou databází
                    int id = _mySql.OhodnotitSpot(s.ID, hodnoceni, IdUzivatele);
                    _sqlCe.OhodnotitSpot(s.ID, hodnoceni, IdUzivatele, id);

                    // načtou se aktualizované spoty a hodnocení do MySql Listů
                    _mySql.NacteniSpotu();
                    _mySql.NacteniHodnoceni();
                }
                // pokud se jedná o soukromý spot nebo není uživatel připojený k online databízi a je přihlášený
                else if ((!_mySqlPripojeno && IdUzivatele > 0) || (_mySqlPripojeno && _typSpotu == "soukromý"))
                {
                    _sqlCe.OhodnotitSpot(s.ID, hodnoceni, IdUzivatele);
                }

                // načtou se aktualizované spoty a hodnocení do SqlCe Listů
                _sqlCe.NacteniSpotu();
                _sqlCe.NacteniHodnoceni();

                NaplneniDataGridu(); // naplní se datagrid záznamy
                break; // ukončím iteraci
            }

            // Získám změněné celkové hodnocení spotu a zobrazím ho
            foreach (Spot s in _spoty.Where(s => s.ID == _aktualniIDSpotu))
            {
                hodnoceniTextBlock.Text = s.Hodnoceni.ToString() + " / 5";
                break;
            }

            // zobrazím uživatelovo hodnocení
            vlastniHodnoceniTextBlock.Text = "Vaše hodnocení: " + hodnoceni.ToString() + " / 5";

            // vypíše se zpráva o hodnocení spotu
            CustomMessageBox.Show("Spot je ohodnocený\nVaše hodnocení: " + hodnoceni + " / 5", "Hodocení");
        }


        // Vymaže zadané hodnoty ze všech ComboBoxů, a Textboxu na hlavní straně, a nastaví se jejich výchozí hodnota
        private void VymazatVyhledavani(Visual c)
        {
            nazevSpotuTextBox.InitialValue(); // nastavím výchozí hodnotu TextBoxu
            foreach (Visual control in c.GetChildren()) // procházím všechny ovládací prvky v rodiči z parametru
                if (control is ComboBox) // pokud se jedná o ComboBox
                {
                    ComboBox cmb = (ComboBox)control;
                    if (cmb.Items.Count > 0) cmb.Text = "Vyberte"; // nastavím ComboBoxu výchozí hodnotu
                }
        }
        #endregion

        #region Ostatní události - TextBoxy

        // volají se metody po přidání / odebrání focusu na textboxy
        private void TextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnGotFocus();
        }

        private void TextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnLostFocus();
        }
        #endregion
    }
}
