﻿using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace DatabazeParkourovychSpotu
{
    /// <summary>
    /// Interaction logic for PrihlaseniRegistrace.xaml
    /// </summary>
    /// 

    public partial class PrihlaseniRegistrace : Window
    {
        private MainWindow _mainWindow;
        public PrihlaseniRegistrace(MainWindow mw)
        {
            InitializeComponent();
            _mainWindow = mw;
        }

        // button pro zaregistrování nového uživatele
        // kontroluje se správné zadání údajů, když jsou správné, tak se uživatel přidá do databáze
        private void registraceButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = _mainWindow.ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                bool registraceOk = true;
                TextBox jmeno = registraceJmenoTextBox;
                TextBox email = registraceEmailTextBox;
                PasswordBox heslo = registraceHesloPasswordBox;
                PasswordBox heslo2 = registracePotvrzeniHeslaPasswordBox;
                if (jmeno.Text.Length < 3 || jmeno.Foreground.ToString() == "#FF606060")
                {
                    registraceOk = false;
                    CustomMessageBox.Show("Nelze se registrovat, jméno musí mít alespoň 3 znaky", "Registrace");
                    jmeno.Focus();
                }
                else
                if ((email.Text.Length < 6 || (!email.Text.Contains("@") || !email.Text.Contains("."))) || email.Foreground.ToString() == "#FF606060")
                {
                    registraceOk = false;
                    CustomMessageBox.Show("Nelze se registrovat, špatně vyplněný e-mail.", "Registrace");
                    email.Focus();
                }
                else
                if (heslo.Password.Length < 4)
                {
                    registraceOk = false;
                    CustomMessageBox.Show("Nelze se registrovat, heslo musí mí alespoň 4 znaky.", "Registrace");
                    heslo.Focus();
                }
                else
                if (heslo.Password != heslo2.Password)
                {
                    registraceOk = false;
                    CustomMessageBox.Show("Nelze se registrovat, hesla se neshodují.", "Registrace");
                    heslo.Focus();
                }
                else
                {
                    if (_mainWindow._mySqlPripojeno)
                    {
                        registraceOk = SqlCeDatabazeUzivateleRegistrace(jmeno, email);
                        if (registraceOk) MySqlDatabazeUzivateleRegistrace(jmeno, email);
                    }
                    else
                    {
                        registraceOk = SqlCeDatabazeUzivateleRegistrace(jmeno, email);
                    }
                }

                if (registraceOk)
                {
                    if (_mainWindow._mySqlPripojeno)
                    {
                        var id = _mainWindow._mySql.RegistrovatSe(jmeno.Text, email.Text, heslo.Password);
                        _mainWindow._sqlCe.RegistrovatSe(jmeno.Text, email.Text, heslo.Password, id);
                        _mainWindow._mySql.NacteniUzivatelu();
                    }
                    else
                    {
                        _mainWindow._sqlCe.RegistrovatSe(jmeno.Text, email.Text, heslo.Password);
                    }

                    _mainWindow._sqlCe.NacteniUzivatelu();
                    CustomMessageBox.Show("Děkujeme, nyní jste zaregistrováni.\nMůžete se přihlásit", "Registrace");
                    heslo.Password = "";
                    heslo2.Password = "";
                    heslo.OnLostFocus();
                    heslo2.OnLostFocus();
                    jmeno.InitialValue();
                    email.InitialValue();
                    prihlaseniJmenoEmailTextBox.Focus();
                }
            }
            else
            {
                this.Close();
            }
        }

        // Metoda zjišťuje jestli již neexistuje stejný uživatel nebo email v SqlCe databázi
        private bool SqlCeDatabazeUzivateleRegistrace(TextBox jmeno, TextBox email)
        {
            bool registraceOk = true;
            int uzivatelIndex = (SqlCeDatabaze.Uzivatele).FindIndex(u => u.UzivatelskeJmeno.ToLower() == jmeno.Text.ToLower());
            int emailIndex = (SqlCeDatabaze.Uzivatele).FindIndex(u => u.Email.ToLower() == email.Text.ToLower());
            if (uzivatelIndex > -1 || emailIndex > -1)
            {
                registraceOk = false;
                CustomMessageBox.Show("Nelze se registrovat, uživatel se stejným jménem nebo emailem již existuje.", "Registrace");
            }
            return registraceOk;
        }

        // Metoda zjišťuje jestli již neexistuje stejný uživatel nebo email v MySql databázi
        private bool MySqlDatabazeUzivateleRegistrace(TextBox jmeno, TextBox email)
        {
            bool registraceOk = true;
            int uzivatelIndex = (MySqlDatabaze.Uzivatele).FindIndex(u => u.UzivatelskeJmeno.ToLower() == jmeno.Text.ToLower());
            int emailIndex = (MySqlDatabaze.Uzivatele).FindIndex(u => u.Email.ToLower() == email.Text.ToLower());
            if (uzivatelIndex > -1 || emailIndex > -1)
            {
                registraceOk = false;
                CustomMessageBox.Show("Nelze se registrovat, uživatel se stejným jménem nebo emailem již existuje.", "Registrace");
            }
            return registraceOk;
        }



        // button pro přihlášení uživatele
        // kontroluje se správné zadání údajů, když jsou správné, tak se nastaví vlastnost Uzivatel a IdUzivatele na zadané jméno, zavře se okno a uživatel je přihlášený
        private void prihlaseniButton_Click(object sender, RoutedEventArgs e)
        {
            bool zmenaPripojeni = _mainWindow.ZmenaPripojeniInternetu();
            if (!zmenaPripojeni)
            {
                bool prihlaseniOk = true;
                string jmeno = "";
                int id = 0;
                TextBox jmenoEmail = prihlaseniJmenoEmailTextBox;
                PasswordBox heslo = prihlaseniHesloPasswordBox;

                if (jmenoEmail.Text.Length == 0 || jmenoEmail.Foreground.ToString() == "#FF606060")
                {
                    prihlaseniOk = false;
                    CustomMessageBox.Show("Vyplňte prosím vaše jméno / e-mail.", "Registrace");
                    jmenoEmail.Focus();
                }
                else
                if (heslo.Password.Length == 0)
                {
                    prihlaseniOk = false;
                    CustomMessageBox.Show("Vyplňte prosím vaše heslo.", "Registrace");
                    heslo.Focus();
                }

                if (prihlaseniOk)
                {
                    if (_mainWindow._mySqlPripojeno)
                    {
                        MySqlDatabaze db = new MySqlDatabaze();
                        db.PrihlasitSe(jmenoEmail.Text, heslo.Password, ref prihlaseniOk, ref jmeno, ref id);
                    }
                    else
                    {
                        SqlCeDatabaze db = new SqlCeDatabaze();
                        db.PrihlasitSe(jmenoEmail.Text, heslo.Password, ref prihlaseniOk, ref jmeno, ref id);

                    }

                    if (prihlaseniOk)
                    {
                        _mainWindow.IdUzivatele = id;
                        _mainWindow.Uzivatel = jmeno;

                        this.Close();
                        CustomMessageBox.Show("Děkujeme, přihlášení bylo úspěšné.\nNyní jste přihlášení.", "Přihlášení");
                        Keyboard.ClearFocus();
                    }
                    else CustomMessageBox.Show("Nelze se přihlásit, nesprávné přihlašovací údaje.", "Přihlášení");
                }
            }
            else
            {
                this.Close();
            }
        }

        private void PasswordBox_LostFocus(object sender, RoutedEventArgs e)
        {
            ((PasswordBox)sender).OnLostFocus();
        }

        private void TextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnGotFocus();
        }

        private void TextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            ((TextBox)sender).OnLostFocus();
        }

        // nastavuje chování prvků při stisknutí enteru
        private async void prihlaseniRegistraceWindow_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (prihlaseniJmenoEmailTextBox.IsFocused || prihlaseniHesloPasswordBox.IsFocused)
            {
                if ((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return)))
                {
                    prihlaseniButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    prihlaseniButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                    prihlaseniButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                    await Task.Delay(200);

                    prihlaseniButton.ClearValue(Button.BackgroundProperty);
                    prihlaseniButton.ClearValue(Button.BorderBrushProperty);
                    return;
                }
            }

            if (registraceJmenoTextBox.IsFocused || registraceEmailTextBox.IsFocused || registraceHesloPasswordBox.IsFocused || registracePotvrzeniHeslaPasswordBox.IsFocused)
            {
                if ((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return)))
                {
                    registraceButton.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                    registraceButton.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                    registraceButton.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                    await Task.Delay(200);

                    registraceButton.ClearValue(Button.BackgroundProperty);
                    registraceButton.ClearValue(Button.BorderBrushProperty);
                    return;
                }

            }
            if (e.Key.Equals(Key.Escape))
            {
                this.Close();
                return;
            }

            foreach (Visual v in hlavniGrid.GetChildren())
            {
                if (v is Button)
                {
                    Button button = (Button)v;
                    if (((e.Key.Equals(Key.Enter)) || (e.Key.Equals(Key.Return))) && button.IsFocused)
                    {
                        button.Background = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));
                        button.BorderBrush = new SolidColorBrush(Color.FromArgb(255, 0, 48, 87));

                        await Task.Delay(200);

                        button.ClearValue(Button.BackgroundProperty);
                        button.ClearValue(Button.BorderBrushProperty);
                    }
                }
            }
        }
    }
}
