﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlServerCe;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;

namespace DatabazeParkourovychSpotu
{
    public class SqlCeDatabaze
    {

        #region Proměnné, vlastnosti

        private SqlCeConnection _pripojeni;
        private SqlCeDataAdapter _dataAdapter;
        private SqlCeDataReader _dataReader;
        private SqlCeCommand _prikaz;
        private string _pripojeniString = Properties.Settings.Default.DpscdbOfflineConnectionString;
        public static List<Spot> Spoty { get; private set; } = new List<Spot>();
        public static List<Obrazek> Obrazky { get; private set; } = new List<Obrazek>();
        public static List<Oblibene> Oblibene { get; private set; } = new List<Oblibene>();
        public static List<Hodnoceni> Hodnoceni { get; private set; } = new List<Hodnoceni>();
        public static List<Uzivatel> Uzivatele { get; private set; } = new List<Uzivatel>();

        #endregion

        #region Inicializace

        // Konstruktor, zavolá metodu k vytvoření připojení a uloží ji do proměnné _pripojeni, poté volá metodu pro naplňění listů
        public SqlCeDatabaze()
        {
            _pripojeni = VytvorPripojeni();
            NacteniListu();
        }

        // vrátí do konstruktoru do proměnné _pripojeni SqlCeConnection připojení
        private SqlCeConnection VytvorPripojeni()
        {
            return new SqlCeConnection(_pripojeniString);
        }

        #endregion

        #region Načtení listů

        // Volají se všechny metody pro načtení záznamů do listů
        private void NacteniListu()
        {
            NacteniSpotu();
            NacteniObrazku();
            NacteniHodnoceni();
            NacteniOblibene();
            NacteniUzivatelu();
        }

        // Načítá spoty do listu
        private void SpotyDoListu()
        {
            int id, idUzivatele;
            string url, nazev, popis, castPrahy, adresa, metro, bus, tram, mapa, streetview, panorama, velikost, povrch, typ, obtiznost, obrazek, zverejneni, typSpotu;
            float hodnoceni;
            using (_prikaz = new SqlCeCommand("SELECT s.id, url, nazev, popis, praha, adresa,metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, typ_spotu, s.id_uzivatele, s.zverejneni, CAST(COALESCE(AVG(h.hodnoceni), 0) as numeric(10,2)) as [hodnoceni], MIN(nahled) as [obrazek] FROM spoty s LEFT OUTER JOIN hodnoceni h ON s.id = h.id_spotu LEFT OUTER JOIN obrazky o ON s.id = o.id_spotu GROUP BY s.id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, typ_treninku, typ_spotu, obtiznost, s.zverejneni, s.id_uzivatele ORDER BY nazev ASC", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Spoty.Count > 0) Spoty.Clear();
                    while (_dataReader.Read())
                    {
                        id = Convert.ToInt32(_dataReader["id"]);
                        url = _dataReader["url"].ToString();
                        nazev = _dataReader["nazev"].ToString();
                        popis = _dataReader["popis"].ToString();
                        castPrahy = _dataReader["praha"].ToString();
                        adresa = _dataReader["adresa"].ToString();
                        metro = _dataReader["metro"].ToString();
                        bus = _dataReader["bus"].ToString();
                        tram = _dataReader["tramvaj"].ToString();
                        mapa = _dataReader["mapa"].ToString();
                        streetview = _dataReader["streetview"].ToString();
                        panorama = _dataReader["panorama"].ToString();
                        velikost = _dataReader["velikost"].ToString();
                        povrch = _dataReader["povrch"].ToString();
                        typ = _dataReader["typ_treninku"].ToString();
                        obtiznost = _dataReader["obtiznost"].ToString();
                        hodnoceni = (float)Convert.ToDecimal(_dataReader["hodnoceni"]);
                        obrazek = _dataReader["obrazek"].ToString();
                        zverejneni = _dataReader["zverejneni"].ToString();
                        typSpotu = _dataReader["typ_spotu"].ToString();
                        idUzivatele = Convert.ToInt32(_dataReader["id_uzivatele"]);

                        Spoty.Add(new Spot(id, url, nazev, popis, castPrahy, adresa, metro, bus, tram, mapa, streetview, panorama, velikost, povrch, typ, obtiznost, hodnoceni, obrazek, zverejneni, typSpotu, idUzivatele));
                    }
                }
            }
        }

        // Načítá obrázky do listu
        private void ObrazkyDoListu()
        {
            using (_prikaz = new SqlCeCommand("SELECT id, id_spotu, nahled, obrazek, typ_obrazku, id_uzivatele FROM obrazky ORDER BY nahled ASC", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Obrazky.Count > 0) Obrazky.Clear();
                    while (_dataReader.Read())
                        Obrazky.Add(new Obrazek(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), _dataReader["nahled"].ToString(), _dataReader["obrazek"].ToString(), _dataReader["typ_obrazku"].ToString(), Convert.ToInt32(_dataReader["id_uzivatele"])));
                }
            }
        }

        // Načítá hodnocení do listu
        private void HodnoceniDoListu()
        {
            using (_prikaz = new SqlCeCommand("SELECT id, id_spotu, hodnoceni, id_uzivatele, zverejneni, sqlCe, mySql FROM hodnoceni", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Hodnoceni.Count > 0) Hodnoceni.Clear();
                    while (_dataReader.Read())
                        Hodnoceni.Add(new Hodnoceni(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), Convert.ToInt32(_dataReader["hodnoceni"]), Convert.ToInt32(_dataReader["id_uzivatele"]), _dataReader["zverejneni"].ToString(), Convert.ToInt32(_dataReader["sqlCe"]), Convert.ToInt32(_dataReader["mySql"])));
                }
            }
        }

        // Načítá oblíbené do listu
        private void OblibeneDoListu()
        {
            using (_prikaz = new SqlCeCommand("SELECT id, id_spotu, id_uzivatele, zverejneni, sqlCe, mySql FROM oblibene WHERE NOT(sqlCe = 0 AND sqlCe = 1)", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Oblibene.Count > 0) Oblibene.Clear();
                    while (_dataReader.Read())
                        Oblibene.Add(new Oblibene(Convert.ToInt32(_dataReader["id"]), Convert.ToInt32(_dataReader["id_spotu"]), Convert.ToInt32(_dataReader["id_uzivatele"]), _dataReader["zverejneni"].ToString(), Convert.ToInt32(_dataReader["sqlCe"]), Convert.ToInt32(_dataReader["mySql"])));
                }
            }
        }

        // Načítá uživatele do listu
        private void UzivateleDoListu()
        {
            using (_prikaz = new SqlCeCommand("SELECT id, url, uzivatel, email, heslo, obrazek, datum, aktivace, stav, sqlCe, mySql FROM uzivatele", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader())
                {
                    if (Uzivatele.Count > 0) Uzivatele.Clear();
                    while (_dataReader.Read())
                        Uzivatele.Add(new Uzivatel(Convert.ToInt32(_dataReader["id"]), _dataReader["url"].ToString(), _dataReader["uzivatel"].ToString(), _dataReader["email"].ToString(), _dataReader["heslo"].ToString(), _dataReader["obrazek"].ToString(), _dataReader["datum"].ToString(), _dataReader["aktivace"].ToString(), Convert.ToInt32(_dataReader["stav"]), Convert.ToInt32(_dataReader["sqlCe"]), Convert.ToInt32(_dataReader["mySql"])));
                }
            }
        }

        // Metody NacteniXXX (XXX = záznam) volají metody pro načtení záznamů do listu
        // Podle stavu připojení buď otevřou nové připjení nebo použijí současné připojení
        public void NacteniSpotu()
        {
            if (_pripojeni.State == ConnectionState.Open)
            {
                SpotyDoListu();
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    SpotyDoListu();
                }
            }
        }
        public void NacteniObrazku()
        {
            if (_pripojeni.State == ConnectionState.Open)
            {
                ObrazkyDoListu();
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    ObrazkyDoListu();
                }
            }
        }
        public void NacteniHodnoceni()
        {
            if (_pripojeni.State == ConnectionState.Open)
            {
                HodnoceniDoListu();
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    HodnoceniDoListu();
                }
            }
        }
        public void NacteniOblibene()
        {
            if (_pripojeni.State == ConnectionState.Open)
            {
                OblibeneDoListu();
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    OblibeneDoListu();
                }
            }
        }
        public void NacteniUzivatelu()
        {
            if (_pripojeni.State == ConnectionState.Open)
            {
                UzivateleDoListu();
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    UzivateleDoListu();
                }
            }
        }

        #endregion

        #region Synchronizace dat


        // SynchronizaceDat - Volají se metody pro synchronizaci záznamů v MySql a SqlCe databázích a vytváří se náhledy ke spotům, kde nejsou
        public void SynchronizaceDat(MySqlDatabaze mySql)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                MySqlToSqlCeSpoty(mySql);
                MySqlToSqlCeObrazky(mySql);
                MySqlToSqlCeHodnoceni(mySql);
                MySqlToSqlCeOblibene(mySql);
                MySqlToSqlCeUzivatele(mySql);
                VytvoreniNahledu();
            }
        }

        // slouží pro překopírování spotů z MySql do SqlCe databáze
        private void MySqlToSqlCeSpoty(MySqlDatabaze mySql)
        {
            bool spotyFetch = false;

            // vyberu nestejné záznamy obou listů a použiji záznam (spot) z MySql databáze
            List<Spot> noveSpoty = MySqlDatabaze.Spoty.Where(x => !Spoty.Any(x1 => x1.Nazev == x.Nazev && x1.Popis == x.Popis && x1.Praha == x.Praha && x1.Adresa == x.Adresa && x1.Metro == x.Metro && x1.Bus == x.Bus && x1.Tramvaj == x.Tramvaj && x1.Mapa == x.Mapa && x1.Streetview == x.Streetview && x1.Panorama == x.Panorama && x1.Velikost == x.Velikost && x1.Obtiznost == x.Obtiznost && x1.Povrch == x.Povrch && x1.Typ == x.Typ)).ToList();

            // procházím nově vytvořený list s nestejnými záznamy
            foreach (Spot spot in noveSpoty)
            {
                spotyFetch = true;

                // Zde se spot z MySql databáze překopíruje i do SqlCe databáze
                using (_prikaz = new SqlCeCommand("INSERT INTO spoty(id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, zverejneni, typ_spotu, id_uzivatele) VALUES(@id, @url, @nazev, @popis, @praha, @adresa, @metro, @bus, @tramvaj, @mapa, @streetview, @panorama, @velikost, @povrch, @obtiznost, @typTreninku, @zverejneni, 'veřejný', -1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", spot.ID).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@url", spot.Url).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@nazev", spot.Nazev).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@popis", spot.Popis).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@praha", spot.Praha).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@adresa", spot.Adresa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@metro", spot.Metro).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@bus", spot.Bus).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@tramvaj", spot.Tramvaj).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@mapa", spot.Mapa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@streetview", spot.Streetview).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@panorama", spot.Panorama).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@velikost", spot.Velikost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@povrch", spot.Povrch).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obtiznost", spot.Obtiznost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@typTreninku", spot.Typ).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(spot.Zverejneni)).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
            if (spotyFetch) NacteniSpotu();
        }

        // slouží pro překopírování obrázků z MySql do SqlCe databáze
        private void MySqlToSqlCeObrazky(MySqlDatabaze mySql)
        {
            bool obrazkyFetch = false;
            List<Obrazek> noveObrazky = MySqlDatabaze.Obrazky.Where(x => !Obrazky.Any(x1 => x1.Nahled == x.Nahled && x1.Fotka == x.Fotka)).ToList();
            foreach (Obrazek obrazek in noveObrazky)
            {
                obrazkyFetch = true;
                using (_prikaz = new SqlCeCommand("INSERT INTO obrazky(id, id_spotu, nahled, obrazek, typ_obrazku, id_uzivatele) VALUES(@id, @idSpotu, @nahled, @obrazek, 'veřejný', -1)", _pripojeni))
                {

                    _prikaz.Parameters.AddWithValue("@id", obrazek.ID).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", obrazek.IdSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@nahled", obrazek.Nahled).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek.Fotka).DbType = DbType.String;
                    _prikaz.ExecuteNonQuery();
                }
            }
            if (obrazkyFetch) NacteniObrazku();
        }

        // slouží pro překopírování hodnocení z MySql do SqlCe
        private void MySqlToSqlCeHodnoceni(MySqlDatabaze mySql)
        {
            bool hodnoceniFetch = false;
            List<Hodnoceni> noveHodnoceni = MySqlDatabaze.Hodnoceni.Where(x => !Hodnoceni.Any(x1 => x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele && x1.HodnoceniSpotu == x.HodnoceniSpotu)).ToList();
            foreach (Hodnoceni hodnoceni in noveHodnoceni)
            {
                hodnoceniFetch = true;
                using (_prikaz = new SqlCeCommand("INSERT INTO hodnoceni(id, id_spotu, hodnoceni, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @hodnoceni, @idUzivatele, @zverejneni, 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", hodnoceni.ID).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", hodnoceni.IdSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni.HodnoceniSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", hodnoceni.IdUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(hodnoceni.Zverejneni)).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
            if (hodnoceniFetch) NacteniHodnoceni();
        }

        // slouží pro překopírování oblíbených z MySql do SqlCe
        private void MySqlToSqlCeOblibene(MySqlDatabaze mySql)
        {
            bool oblibeneFetch = false;
            List<Oblibene> noveOblibene = MySqlDatabaze.Oblibene.Where(x => !Oblibene.Any(x1 => (x1.IdSpotu == x.IdSpotu && x1.IdUzivatele == x.IdUzivatele))).ToList();
            foreach (Oblibene oblibene in noveOblibene)
            {
                oblibeneFetch = true;
                using (_prikaz = new SqlCeCommand("INSERT INTO oblibene(id, id_spotu, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @idUzivatele, @zverejneni, 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", oblibene.ID).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", oblibene.IdSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", oblibene.IdUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Parse(oblibene.Zverejneni)).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
            if (oblibeneFetch) NacteniOblibene();
        }

        // slouží pro překopírování uživatelů z MySql do SqlCe
        private void MySqlToSqlCeUzivatele(MySqlDatabaze mySql)
        {
            bool uzivateleFetch = false;
            // vybírám uživatele, kteří nejsou v SqlCe, ale jsou v MySql, ale už nevybírám uživatele, kteří jsou v obou
            // jelikož to se vyřešilo už v MySqlDatabaze třídě
            // když takovéto uživatele najdu, tak je z MySql databáze překopíruji do SqlCe databáze
            List<Uzivatel> novyUzivatele = MySqlDatabaze.Uzivatele.Where(x => !Uzivatele.Any(x1 => x1.UzivatelskeJmeno == x.UzivatelskeJmeno && x1.Email == x.Email && x1.Heslo == x.Heslo && x1.Obrazek == x.Obrazek && x1.Stav == x.Stav)).ToList();
            foreach (Uzivatel uzivatel in novyUzivatele)
            {
                uzivateleFetch = true;
                using (_prikaz = new SqlCeCommand("INSERT INTO uzivatele(id, url, uzivatel, email, heslo, obrazek, datum, aktivace, stav, sqlCe, mySql) VALUES(@id, @url, @uzivatel, @email, @heslo, @obrazek, @datum, @aktivace, @stav, 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", uzivatel.ID).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@url", uzivatel.Url).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel.UzivatelskeJmeno).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@email", uzivatel.Email).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@heslo", uzivatel.Heslo).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", uzivatel.Obrazek).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@datum", DateTime.Parse(uzivatel.Datum)).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@aktivace", uzivatel.Aktivace).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@stav", uzivatel.Stav).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
            if (uzivateleFetch) NacteniUzivatelu();
        }

        #endregion

        #region Vytváření záznamů


        // vytvoří se nový spot v SqlCe databázi
        public int VytvoreniSpotu(string url, string nazev, string popis, string praha, string adresa, string metro, string bus, string tramvaj, string mapa, string streetview, string panorama, string velikost, string povrch, string obtiznost, string typTreninku, DateTime zverejneni, string typSpotu, int idUzivatele)
        {
            int id = -1;
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                id = ZiskaniId("spoty");
                using (_prikaz = new SqlCeCommand("INSERT INTO spoty(id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, zverejneni, typ_spotu, id_uzivatele) VALUES(@id, @url, @nazev, @popis, @praha, @adresa, @metro, @bus, @tramvaj, @mapa, @streetview, @panorama, @velikost, @povrch, @obtiznost, @typTreninku, @zverejneni, @typSpotu, @idUzivatele)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@url", url).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@nazev", nazev).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@popis", popis).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@praha", praha).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@adresa", adresa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@metro", metro).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@bus", bus).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@tramvaj", tramvaj).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@mapa", mapa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@streetview", streetview).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@panorama", panorama).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@velikost", velikost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@povrch", povrch).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obtiznost", obtiznost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@typTreninku", typTreninku).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@zverejneni", zverejneni).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@typSpotu", typSpotu).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
            return id;
        }

        // vytvoří se nový obrázek v SqlCe databázi
        public void VytvoreniObrazku(SqlCeDatabaze sqlCe, int idSpotu, string nahled, string obrazek, string typObrazku, int idUzivatele)
        {
            int id = -1;
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                id = ZiskaniId("obrazky");
                using (_prikaz = new SqlCeCommand("INSERT INTO obrazky(id_spotu, nahled, obrazek, typ_obrazku, id_uzivatele) VALUES(@idSpotu, @nahled, @obrazek, @typObrazku, @idUzivatele)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@nahled", nahled).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@typObrazku", typObrazku).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        #endregion

        #region Aktualizace záznamů

        // Tyto metody aktualizují záznamy, nejdříve se smaží a poté se vloží do databáze s novými údaji
        public void UpdateUzivatele(int id, string url, string uzivatel, string email, string heslo, string obrazek, DateTime datum, string aktivace, int stav, int sqlCe, int mySql)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("DELETE FROM uzivatele WHERE uzivatel = @uzivatel AND heslo = @heslo", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@heslo", heslo).DbType = DbType.String;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("INSERT INTO uzivatele(id, url, uzivatel, email, heslo, obrazek, datum, aktivace, stav, sqlCe, mySql) VALUES(@id, @url, @uzivatel, @email, @heslo, @obrazek, @datum, @aktivace, @stav, @sqlCe, @mySql)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@url", url).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@email", email).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@heslo", heslo).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@datum", datum).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@aktivace", aktivace).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@stav", stav).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@sqlCe", sqlCe).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@mySql", mySql).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }
        public void UpdateHodnoceni(int id, int idSpotu, int hodnoceni, int idUzivatele, DateTime zverejneni, int sqlCe, int mySql)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("DELETE FROM hodnoceni WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("INSERT INTO hodnoceni(id, id_spotu, hodnoceni, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @hodnoceni, @idUzivatele, @zverejneni, @sqlCe, @mySql)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", zverejneni).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@sqlCe", sqlCe).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@mySql", mySql).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }
        public void UpdateOblibene(int id, int idSpotu, int idUzivatele, DateTime zverejneni, int sqlCe, int mySql)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("DELETE FROM oblibene WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("INSERT INTO oblibene(id, id_spotu, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @idUzivatele, @zverejneni, @sqlCe, @mySql)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", zverejneni).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@sqlCe", sqlCe).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@mySql", mySql).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }
        public void UpdateSpoty(int id, string url, string nazev, string popis, string praha, string adresa, string metro, string bus, string tramvaj, string mapa, string streetview, string panorama, string velikost, string povrch, string obtiznost, string typTreninku, DateTime zverejneni, string typSpotu, int idUzivatele, string zmena)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                string query = "DELETE FROM spoty WHERE nazev = @nazev AND popis = @popis AND praha = @praha AND adresa = @adresa AND mapa = @mapa";
                if (zmena == "ID")
                {
                    query = "DELETE FROM spoty WHERE id = @id";
                }

                using (_prikaz = new SqlCeCommand(query, _pripojeni))
                {
                    if (zmena == "ID")
                    {
                        _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    }
                    else
                    {
                        _prikaz.Parameters.AddWithValue("@nazev", nazev).DbType = DbType.String;
                        _prikaz.Parameters.AddWithValue("@popis", popis).DbType = DbType.String;
                        _prikaz.Parameters.AddWithValue("@praha", praha).DbType = DbType.String;
                        _prikaz.Parameters.AddWithValue("@adresa", adresa).DbType = DbType.String;
                        _prikaz.Parameters.AddWithValue("@mapa", mapa).DbType = DbType.String;
                    }
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("INSERT INTO spoty(id, url, nazev, popis, praha, adresa, metro, bus, tramvaj, mapa, streetview, panorama, velikost, povrch, obtiznost, typ_treninku, zverejneni, typ_spotu, id_uzivatele) VALUES(@id, @url, @nazev, @popis, @praha, @adresa, @metro, @bus, @tramvaj, @mapa, @streetview, @panorama, @velikost, @povrch, @obtiznost, @typTreninku, @zverejneni, @typSpotu, @idUzivatele)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@url", url).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@nazev", nazev).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@popis", popis).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@praha", praha).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@adresa", adresa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@metro", metro).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@bus", bus).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@tramvaj", tramvaj).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@mapa", mapa).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@streetview", streetview).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@panorama", panorama).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@velikost", velikost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@povrch", povrch).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obtiznost", obtiznost).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@typTreninku", typTreninku).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@zverejneni", zverejneni).DbType = DbType.DateTime;
                    _prikaz.Parameters.AddWithValue("@typSpotu", typSpotu).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }
        public void UpdateObrazky(int id, int idSpotu, string nahled, string obrazek, string typObrazku, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("DELETE FROM obrazky WHERE id_spotu = @idSpotu AND nahled = @nahled AND obrazek = @obrazek", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@nahled", nahled).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek).DbType = DbType.String;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("INSERT INTO obrazky(id, id_spotu, nahled, obrazek, typ_obrazku, id_uzivatele) VALUES(@id, @idSpotu, @nahled, @obrazek, @typObrazku, @idUzivatele)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@nahled", nahled).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@obrazek", obrazek).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@typObrazku", typObrazku).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        #endregion

        #region Smazávání záznamů

        // Maže záznam podle předaného ID a tabulky v parametrech
        public void DeleteZaznam(int id, string tabulka)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("DELETE FROM " + tabulka + " WHERE id = " + id, _pripojeni))
                {
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // Maže všechny přidružené záznamy ke spotu
        public void SmazaniPridruzenychZaznamuKeSpotu(int idSpotu, string urlSpotu)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("DELETE FROM obrazky WHERE id_spotu = @idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("SELECT MIN(nahled) as [obrazek] FROM obrazky WHERE id_spotu=@idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    using (_dataReader = _prikaz.ExecuteReader()) while (_dataReader.Read()) SmazaniNahledu(urlSpotu);
                }

                using (_prikaz = new SqlCeCommand("DELETE FROM oblibene WHERE id_spotu = @idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }

                using (_prikaz = new SqlCeCommand("DELETE FROM hodnoceni WHERE id_spotu = @idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        #endregion

        #region Náhledy, složky

        // Vrací TRUE, pokud existují všechny náhledy, jinak FALSE
        public bool ExistujiNahledy()
        {
            string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
            var fileCount = (from file in Directory.EnumerateFiles(baseDirectory + "img\\spoty\\nahledy", "*.*", SearchOption.AllDirectories)
                             select file).Count();

            if ((Obrazky.Count != fileCount) || (!Directory.Exists(baseDirectory + "img\\spoty\\nahledy\\")))
            {
                return false;
            }
            return true;
        }

        // Pokud spoty mají mít náhledy, ale náhledy neexistují, tak se náhledy stáhnou a vytvoří
        private void VytvoreniNahledu()
        {
            string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
            SmazaniPrazdnychSlozek(baseDirectory + "img\\spoty\\nahledy\\");

            foreach (Obrazek obrazek in Obrazky)
            {
                string slozka = obrazek.Nahled.Substring(0, obrazek.Nahled.LastIndexOf("/"));
                slozka = baseDirectory + "img\\spoty\\nahledy\\" + slozka;
                string soubor = obrazek.Nahled.Substring(obrazek.Nahled.LastIndexOf("/") + 1);
                string urlSouboru = "http://www.dpsc.cz/img/spoty/nahledy/" + obrazek.Nahled;

                Directory.CreateDirectory(@slozka);
                if (!File.Exists(@slozka + "\\" + soubor)) using (WebClient wc = new WebClient()) wc.DownloadFile(urlSouboru, @slozka + "\\" + soubor);
            }
        }

        // Maže náhled ke spotu podle zadané url spotu
        private void SmazaniNahledu(string urlSpotu)
        {
            string baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
            string slozka = baseDirectory + "img\\spoty\\nahledy\\" + urlSpotu;
            Directory.Delete(slozka, true);
        }
        private void SmazaniPrazdnychSlozek(string vychoziSlozka)
        {
            if (!(Directory.Exists(vychoziSlozka) || File.Exists(vychoziSlozka)))
            {
                Directory.CreateDirectory(vychoziSlozka);
            }
            foreach (var slozka in Directory.GetDirectories(vychoziSlozka))
            {
                SmazaniPrazdnychSlozek(slozka);
                if (Directory.GetFiles(slozka).Length == 0 && Directory.GetDirectories(slozka).Length == 0) Directory.Delete(slozka, false);
            }
        }

        #endregion

        #region Oblíbené, hodnocené

        // přidá do tabulky oblibene uživatelův oblíbený spot, tato metoda se volá, když je aplikace v online režimu
        public void PridaniSpotuDoOblibenych(int idSpotu, int idUzivatele, int id)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("INSERT INTO oblibene(id, id_spotu, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @idUzivatele, @zverejneni, 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // přidá do tabulky oblibene uživatelův oblíbený spot, tato metoda se volá, když je aplikace v offline režimu
        public void PridaniSpotuDoOblibenych(int idSpotu, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                int id = ZiskaniId("oblibene");
                using (_prikaz = new SqlCeCommand("INSERT INTO oblibene(id, id_spotu, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @idUzivatele, @zverejneni, 1, 0)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // odebere uživatelovo oblíbené z databáze
        public void OdebraniSpotuZOblibenych(int idSpotu, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("DELETE FROM oblibene WHERE id_spotu = " + idSpotu + " AND id_uzivatele = " + idUzivatele, _pripojeni))
                    _prikaz.ExecuteNonQuery();
            }
        }

        // aktualizuje oblíbené tak, aby se při příští synchronizaci dat smazalo
        public void PripravaOdebraniOblibene(int idSpotu, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                int id = 0, sqlCe = 0, mySql = 0;

                _pripojeni.Open();

                using (_prikaz = new SqlCeCommand("SELECT id, sqlCe, mySql FROM oblibene WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read())
                        {
                            id = Convert.ToInt32(_dataReader["id"].ToString());
                            sqlCe = (Convert.ToBoolean(_dataReader["sqlCe"])) ? 1 : 0;
                            mySql = (Convert.ToBoolean(_dataReader["mySql"])) ? 1 : 0;
                        }
                }

                if (sqlCe == 1 && mySql == 0)
                {
                    DeleteZaznam(id, "oblibene");
                }
                else
                {
                    using (_prikaz = new SqlCeCommand("UPDATE oblibene SET sqlCe = 0, mySql = 1 WHERE id_spotu = @idSpotu AND id_uzivatele = @idUzivatele", _pripojeni))
                    {
                        _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                        _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                        _prikaz.ExecuteNonQuery();
                    }
                }
            }
        }

        // přidá hodnocení do databáze, online aplikace
        public void OhodnotitSpot(int idSpotu, int hodnoceni, int idUzivatele, int id)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("INSERT INTO hodnoceni(id, id_spotu, hodnoceni, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @hodnoceni, @idUzivatele, @zverejneni, 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // přidá hodnocení do databáze, offline aplikace
        public void OhodnotitSpot(int idSpotu, int hodnoceni, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                int id = ZiskaniId("hodnoceni");
                using (_prikaz = new SqlCeCommand("INSERT INTO hodnoceni(id, id_spotu, hodnoceni, id_uzivatele, zverejneni, sqlCe, mySql) VALUES(@id, @idSpotu, @hodnoceni, @idUzivatele, @zverejneni, 1, 0)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@hodnoceni", hodnoceni).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@zverejneni", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }


        // načte hodnocení uživatele z databáze, vrátí hodnotu ohodnocení
        public string ZiskatHodnoceni(int idUzivatele, int idSpotu)
        {
            string hodnoceni = "";
            List<string> hodnoceniList = new List<string>();

            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("SELECT id_spotu, hodnoceni, id_uzivatele FROM hodnoceni WHERE id_uzivatele = @idUzivatele AND id_spotu = @idSpotu", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@idUzivatele", idUzivatele).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@idSpotu", idSpotu).DbType = DbType.Int32;
                    using (_dataReader = _prikaz.ExecuteReader()) while (_dataReader.Read()) hodnoceni = _dataReader["hodnoceni"].ToString();
                }
            }
            return hodnoceni;
        }

        #endregion

        #region Registrace, přihlášení

        // Zaregistruje uživatele, vloží ho do databáze, online režim aplikace
        public void RegistrovatSe(string uzivatel, string email, string heslo, int id)
        {
            // zakóduji heslo
            heslo = heslo.HashPassword();
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("INSERT INTO uzivatele(id, uzivatel, url, email, heslo, obrazek, datum, stav, aktivace, sqlCe, mySql) VALUES(@id, @uzivatel, '', @email, @heslo, 'default.png', @datum, 1, '123456789', 1, 1)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.Int32;
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@email", email).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@heslo", heslo).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@datum", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // Zaregistruje uživatele, vloží ho do databáze, offline režim aplikace
        public void RegistrovatSe(string uzivatel, string email, string heslo)
        {
            // zakóduji heslo
            heslo = heslo.HashPassword();
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                int id = ZiskaniId("uzivatele");
                using (_prikaz = new SqlCeCommand("INSERT INTO uzivatele(id, uzivatel, url, email, heslo, obrazek, datum, stav, aktivace, sqlCe, mySql) VALUES(@id, @uzivatel, '', @email, @heslo, 'default.png', @datum, 1, '123456789', 1, 0)", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@id", id).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@uzivatel", uzivatel).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@email", email).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@heslo", heslo).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@datum", DateTime.Now).DbType = DbType.DateTime;
                    _prikaz.ExecuteNonQuery();
                }
            }
        }

        // Zjistí, jestli údaje o přihlášení jsou správné
        public void PrihlasitSe(string jmenoEmail, string heslo, ref bool prihlaseniOk, ref string uzivatel, ref int id)
        {
            // zakóduji heslo
            heslo = heslo.HashPassword();
            prihlaseniOk = false;

            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                using (_prikaz = new SqlCeCommand("SELECT id, email, uzivatel FROM uzivatele WHERE (email = @email OR uzivatel = @jmeno) AND heslo = @heslo", _pripojeni))
                {
                    _prikaz.Parameters.AddWithValue("@jmeno", jmenoEmail).DbType = DbType.String;
                    _prikaz.Parameters.AddWithValue("@email", jmenoEmail).DbType = DbType.String; ;
                    _prikaz.Parameters.AddWithValue("@heslo", heslo).DbType = DbType.String; ;

                    using (_dataReader = _prikaz.ExecuteReader())
                        while (_dataReader.Read())
                        {
                            prihlaseniOk = true;
                            uzivatel = _dataReader["uzivatel"].ToString();
                            id = (int)_dataReader["id"];
                        }
                }
            }
        }

        #endregion

        #region Vybrání výsledků, Vytvoření příkazu SELECT, Načtení záznamů

        // Metoda definuje WHERE podmínku pro vyhledávání a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku(string nazev, string castPrahy, string povrch, string obtiznost, string velikost, string typ, int idUzivatele)
        {
            nazev = (nazev == "Zadejte název spotu" || nazev == "") ? "" : "AND nazev like '%" + nazev + "%'";
            castPrahy = (castPrahy == "Vyberte" || castPrahy == "Nezáleží" || castPrahy == "") ? "" : "AND praha = '" + castPrahy + "'";
            povrch = (povrch == "Vyberte" || povrch == "Nezáleží" || povrch == "") ? "" : "AND povrch = '" + povrch + "'";
            obtiznost = (obtiznost == "Vyberte" || obtiznost == "Nezáleží" || obtiznost == "") ? "" : "AND obtiznost = '" + obtiznost + "'";
            velikost = (velikost == "Vyberte" || velikost == "Nezáleží" || velikost == "") ? "" : "AND velikost = '" + velikost + "'";
            typ = (typ == "Vyberte" || typ == "Nezáleží" || typ == "") ? "" : "AND typ_treninku = '" + typ + "'";
            string typSpotu = (idUzivatele == 0) ? " AND typ_spotu = 'veřejný' " : " AND (typ_spotu = 'veřejný' OR s.id_uzivatele = " + idUzivatele + ") ";
            string where = nazev + " " + castPrahy + " " + obtiznost + " " + velikost + " " + typ + " " + povrch + " " + typSpotu;
            where = " " + where.Trim() + " ";
            if (where.Trim() == "") { where = ""; }
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Metoda definuje WHERE podmínku pro uživatelem ohodnocené spoty nebo uživatelovi oblíbené spoty nebo uživatelovi vlastní spoty a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku(int idUzivatele, string uzivatelskaAkce)
        {
            string typSpotu = (idUzivatele == 0) ? " AND typ_spotu = 'veřejný' " : " AND (typ_spotu = 'veřejný' OR s.id_uzivatele = " + idUzivatele + ") ";
            string where = "";
            switch (uzivatelskaAkce)
            {
                case "hodnocení":
                    where = "AND h.id_uzivatele = " + idUzivatele + " " + typSpotu;
                    break;
                case "oblíbené":
                    where = "AND o.id_uzivatele = " + idUzivatele + " AND NOT(o.sqlCe = 0 AND o.mySql = 1) " + typSpotu;
                    break;
                case "vlastní":
                    where = "AND s.id_uzivatele = " + idUzivatele + " AND typ_spotu = 'soukromý'";
                    break;
            }
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Metoda resetuje WHERE podmínku a vrací SELECT pomocí metody VytvoreniPrikazuVyhledavani()
        public string VybraniVysledku(int idUzivatele)
        {
            string where = (idUzivatele == 0) ? " AND typ_spotu = 'veřejný' " : " AND (typ_spotu = 'veřejný' OR s.id_uzivatele = " + idUzivatele + ") ";
            return VytvoreniPrikazuVyhledavani(where);
        }

        // Vytváří a vrací SELECT pro vyhledávání spotů
        private string VytvoreniPrikazuVyhledavani(string where)
        {
            string select = "SELECT s.id as [ID], nazev as [Název], praha AS [ČástPrahy], povrch AS [Povrch], obtiznost AS [ObtížnostSkoků], velikost as [Velikost], typ_treninku AS [TypTréninku], CAST(COALESCE(AVG(h.hodnoceni), 0) as numeric(10,2)) as [Hodnocení] FROM spoty s LEFT OUTER JOIN hodnoceni h ON s.id = h.id_spotu LEFT OUTER JOIN oblibene o ON s.id = o.id_spotu WHERE 1=1 " + where + " GROUP BY id, nazev, praha, povrch, obtiznost, velikost, typ_treninku ORDER BY Název ASC";
            return select;
        }

        // Načítá data z databáze a podle zvoleného SELECT příkazu vrací dataSet pro překopírování do DataGridu
        public DataSet NacteniZaznamu(string select)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                DataSet ds = new DataSet();
                _dataAdapter = new SqlCeDataAdapter(select, _pripojeni);
                _dataAdapter.Fill(ds, "spoty");
                return ds;
            }
        }

        // Načítá soukromé spoty z databáze a podle zvoleného SELECT příkazu vrací dataSet pro překopírování do DataGridu
        public DataSet NacteniSoukromychSpotuVyhledavani(string select, int idUzivatele)
        {
            _pripojeni = new SqlCeConnection(_pripojeniString);
            using (_pripojeni)
            {
                _pripojeni.Open();
                DataSet ds = new DataSet();
                _dataAdapter = new SqlCeDataAdapter(select, _pripojeni);
                _dataAdapter.Fill(ds, "soukrome_spoty");
                return ds;
            }
        }

        #endregion

        #region Ostatní, získání ID

        // Vybere ID z tabulky
        private int VyberId(string tabulka)
        {
            int id = 0;
            using (_prikaz = new SqlCeCommand("SELECT TOP 1 id FROM " + tabulka + " ORDER BY id DESC", _pripojeni))
            {
                using (_dataReader = _prikaz.ExecuteReader()) while (_dataReader.Read()) id = Convert.ToInt32(_dataReader["id"]);
            }
            return id;
        }

        // Podle zadaného parametru získá nové ID pro záznam
        public int ZiskaniId(string tabulka)
        {
            int id = 0;
            if (_pripojeni.State == ConnectionState.Open)
            {
                id = VyberId(tabulka);
            }
            else
            {
                _pripojeni = new SqlCeConnection(_pripojeniString);
                using (_pripojeni)
                {
                    _pripojeni.Open();
                    id = VyberId(tabulka);
                }
            }
            return id + 1;
        }

        #endregion

    }
}
